import math
import shutil
import tempfile
import time
from os import PathLike
from pathlib import Path
from typing import Union, Dict

import SimpleITK as sitk
import dicom2nifti
import nibabel as nib
import numpy as np
import pydicom
import pydicom_seg
from nilearn.image import resample_to_img
from pydicom import dcmread

from PyMAIA.utils.file_utils import subfolders
from PyMAIA.utils.log_utils import get_logger

logger = get_logger(__name__)


def dcm2nii_CT(CT_dcm_path: Union[str, PathLike], nii_out_path: Union[str, PathLike]):
    """
    Conversion of CT DICOM to nifti and save in nii_out_path.

    Parameters
    ----------
    CT_dcm_path :
        CT DICOM folder path.
    nii_out_path :
        Output NIFTI file path.
    """

    with tempfile.TemporaryDirectory() as tmp:
        tmp = Path(str(tmp))
        dicom2nifti.convert_directory(CT_dcm_path, str(tmp), compression=True, reorient=True)
        nii = next(tmp.glob("*nii.gz"))
        shutil.copy(nii, nii_out_path)


def dcm2nii_seg(mask_dcm_path: Union[str, PathLike], nii_out_path: Union[str, PathLike], ref_nii_path: Union[str]):
    """
    Conversion of SEG DICOM to NIFTI, using a reference NIFTI image.

    Parameters
    ----------
    mask_dcm_path :
        SEG DICOM folder.
    nii_out_path :
        NIFTI file saved as output.
    ref_nii_path :
        Reference NIFTI used to correctly saved the segmentation volume.
    """
    dcm = pydicom.dcmread(mask_dcm_path)

    reader = pydicom_seg.MultiClassReader()
    result = reader.read(dcm)

    image_data = result.data  # directly available
    image = result.image  # lazy construction
    sitk.WriteImage(image, nii_out_path, True)

    input_image = nib.load(nii_out_path)
    reference_image = nib.load(ref_nii_path)
    resampled_image = resample_to_img(input_image, reference_image, fill_value=0)
    nib.save(resampled_image, nii_out_path)

def dcm2nii_mask(mask_dcm_path: Union[str, PathLike], nii_out_path: Union[str, PathLike], ref_nii_path: Union[str, PathLike]):
    """
    Converts a SEG DICOM volume into NIFTI format. Requires an existing NIFTI file to derive the corresponding affine transform.

    Parameters
    ----------
    mask_dcm_path :
        SEG DICOM folder.
    nii_out_path :
        NIFTI file saved as output.
    ref_nii_path :
        Reference NIFTI used to correctly saved the segmentation volume.
    """
    mask_dcm = list(mask_dcm_path.glob("*.dcm"))[0]
    mask = pydicom.read_file(str(mask_dcm))
    mask_array = mask.pixel_array

    mask_array = np.transpose(mask_array, (2, 1, 0))
    mask_orientation = mask[0x5200, 0x9229][0].PlaneOrientationSequence[0].ImageOrientationPatient
    if mask_orientation[4] == -1:
        mask_array = np.flip(mask_array, 1)

    # get affine matrix from the corresponding pet
    pet = nib.load(ref_nii_path)
    pet_affine = pet.affine

    # return mask as nifti object
    mask_out = nib.Nifti1Image(mask_array, pet_affine)
    nib.save(mask_out, nii_out_path)


def convert_DICOM_folder_to_NIFTI_image(patient_dicom_folder: Union[str, PathLike],
                                        patient_nifti_folder: Union[str, PathLike]) -> Dict[str, str]:
    """
    Converts a given Patient DICOM folder into NIFTI format, saving the DICOM Studies in different folders.

    Parameters
    ----------
    patient_dicom_folder :
        DICOM folder containing a single patient Studies.
    patient_nifti_folder :
        Output NIFTI folder used as stem to save the DICOM Studies. The Study index is appended to this path to create
        the corresponding NIFTI study folder path.

    Returns
    -------
    Dictionary mapping each Patient -> Study ID to the corresponding StudyInstanceUID
    """
    studies = subfolders(patient_dicom_folder, join=False)
    single_study = False
    if len(studies) == 1:
        single_study = True

    patient_study_map = {Path(patient_dicom_folder).name: {}}
    for study_id, study in enumerate(studies):
        if not single_study:
            Path(str(patient_nifti_folder) + "_{}".format(study_id)).mkdir(parents=True, exist_ok=True)
        else:
            Path(str(patient_nifti_folder)).mkdir(parents=True, exist_ok=True)
        series = subfolders(Path(patient_dicom_folder).joinpath(study), join=False)
        for serie in series:
            first_file = next(Path(patient_dicom_folder).joinpath(study, serie).glob("*.dcm"))
            ds = pydicom.dcmread(str(first_file))
            patient_study_map[Path(patient_dicom_folder).name][study_id] = str(ds['StudyInstanceUID'].value)

            if Path(patient_dicom_folder).name != ds['PatientName'].value:
                print(
                    f"WARNING! Patient name is different: {ds['PatientName'].value} instead of {Path(patient_dicom_folder).name}")

            if ds.Modality == "CT":
                if not single_study:
                    ct_filename = str(
                        Path(str(patient_nifti_folder) + "_{}".format(study_id)).joinpath(
                            "{}_{}_CT.nii.gz".format(Path(patient_dicom_folder).name, study_id)
                        )
                    )
                else:
                    ct_filename = str(
                        Path(str(patient_nifti_folder)).joinpath("{}_CT.nii.gz".format(Path(patient_dicom_folder).name))
                    )

                dcm2nii_CT(str(Path(patient_dicom_folder).joinpath(study, serie)), ct_filename)
            elif ds.Modality == "PT":
                if not single_study:
                    pet_filename = str(
                        Path(str(patient_nifti_folder) + "_{}".format(study_id)).joinpath(
                            "{}_{}_PET.nii.gz".format(Path(patient_dicom_folder).name, study_id)
                        )
                    )
                else:
                    pet_filename = str(
                        Path(str(patient_nifti_folder)).joinpath("{}_PET.nii.gz".format(Path(patient_dicom_folder).name))
                    )
                normalize_PET_to_SUV_BW(str(Path(patient_dicom_folder).joinpath(study, serie)), pet_filename)

    for study_id, study in enumerate(studies):
        series = subfolders(Path(patient_dicom_folder).joinpath(study), join=False)
        for serie in series:
            first_file = next(Path(patient_dicom_folder).joinpath(study, serie).glob("*.dcm"))
            ds = pydicom.dcmread(str(first_file))

            if ds.Modality == "SEG":
                if not single_study:
                    seg_filename = str(
                        Path(str(patient_nifti_folder) + "_{}".format(study_id)).joinpath(
                            "{}_{}_SEG.nii.gz".format(Path(patient_dicom_folder).name, study_id)
                        )
                    )
                    ref_filename = str(
                        Path(str(patient_nifti_folder) + "_{}".format(study_id)).joinpath(
                            "{}_{}_PET.nii.gz".format(Path(patient_dicom_folder).name, study_id)
                        )
                    )
                else:
                    seg_filename = str(
                        Path(str(patient_nifti_folder)).joinpath(
                            "{}_SEG.nii.gz".format(Path(patient_dicom_folder).name))
                    )
                    ref_filename = str(
                        Path(str(patient_nifti_folder)).joinpath(
                            "{}_PET.nii.gz".format(Path(patient_dicom_folder).name))
                    )
                dcm2nii_mask(Path(patient_dicom_folder).joinpath(study, serie), seg_filename, ref_filename)
    return patient_study_map

def normalize_PET_to_SUV_BW(dicom_pet_series_folder: Union[str, PathLike], suv_pet_filename: Union[str, PathLike]):
    """
    SUV BW Normalization of DICOM PET volume. The resulting normalized PET volume is saved at **suv_pet_filename**.

    Parameters
    ----------
    dicom_pet_series_folder :
        DICOM PET Folder to be normalized.
    suv_pet_filename:
        Normalized SUV PET file location.
    """
    reader = sitk.ImageSeriesReader()
    dicom_names = reader.GetGDCMSeriesFileNames(dicom_pet_series_folder)

    ds = dcmread(dicom_names[0])

    corrected_image = ds[0x0028, 0x0051].value
    decay_correction = ds[0x0054, 0x1102].value
    units = ds[0x0054, 0x1001].value

    series_date = ds.SeriesDate
    acquisition_date = ds.AcquisitionDate
    series_time = ds.SeriesTime
    acquisition_time = ds.AcquisitionTime
    half_life = ds.RadiopharmaceuticalInformationSequence[0].RadionuclideHalfLife
    weight = ds.PatientWeight

    if "ATTN" in corrected_image and "DECY" in corrected_image and decay_correction == "START":
        if units == "BQML":
            if series_time <= acquisition_time and series_date <= acquisition_date:
                scan_date = series_date
                scan_time = series_time
            else:
                scan_date = acquisition_date
                scan_time = acquisition_time
            # if not RadiopharmaceuticalStartTime in ds.RadiopharmaceuticalInformationSequence[0]:
            #    ...
            # else:
            start_time = ds.RadiopharmaceuticalInformationSequence[0].RadiopharmaceuticalStartTime
            start_date = scan_date

            scan_time = str(round(float(scan_time)))
            str_scan_time = time.strptime(scan_date + scan_time, "%Y%m%d%H%M%S")

            start_time = str(round(float(start_time)))

            str_start_time = time.strptime(start_date + start_time, "%Y%m%d%H%M%S")

            decay_time = time.mktime(str_scan_time) - time.mktime(str_start_time)

            injected_dose = ds.RadiopharmaceuticalInformationSequence[0].RadionuclideTotalDose
            decayed_dose = injected_dose * math.pow(2, -decay_time / half_life)

            SUB_BW_scale_factor = (weight * 1000) / decayed_dose

    rescale_slope = 1  # ds[0x0028,0x1053].value

    total_factor = rescale_slope * SUB_BW_scale_factor

    reader.SetFileNames(dicom_names)

    image = reader.Execute()

    image_array = sitk.GetArrayFromImage(image)

    image_array *= total_factor

    itk_image = sitk.GetImageFromArray(image_array)

    itk_image.CopyInformation(image)

    sitk.WriteImage(itk_image, suv_pet_filename)


def resample_image(nii_input_path: Union[str, PathLike], nii_ref_path: Union[str, PathLike],
                   nii_out_path: Union[str, PathLike]):
    """
    Resample Input NIFTI to match Reference NIFTI size and spacing.

    Parameters
    ----------
    nii_input_path  :
        Input NIFTI File Path.
    nii_ref_path    :
        Reference NIFTI File Path.
    nii_out_path
        Resampled Output NIFTI File Path.
    """
    # resample CT to PET and mask resolution
    input_image = nib.load(nii_input_path)
    reference_image = nib.load(nii_ref_path)
    resampled_image = resample_to_img(input_image, reference_image, fill_value=-1024)
    nib.save(resampled_image, nii_out_path)



#!/usr/bin/env python

import json
import os
import subprocess
from argparse import ArgumentParser, RawTextHelpFormatter
from pathlib import Path
from textwrap import dedent

from PyMAIA.utils.log_utils import get_logger, add_verbosity_options_to_argparser, log_lvl_from_verbosity_args, str2bool

DESC = dedent(
    """
        Run ``nnUNetv2_train`` command to start nnUNet training for the specified fold.
    """  # noqa: E501
)
EPILOG = dedent(
    """
    Example call:
    ::
        {filename} --config-file ../CONFIG_FILE.json --run-fold 0
        {filename} --config-file ../CONFIG_FILE.json --run-fold 0 --resume-training y
    """.format(  # noqa: E501
        filename=Path(__file__).name
    )
)


def get_arg_parser():
    pars = ArgumentParser(description=DESC, epilog=EPILOG, formatter_class=RawTextHelpFormatter)

    pars.add_argument(
        "--config-file",
        type=str,
        required=True,
        help="File path for the configuration dictionary, used to retrieve experiments variables (Task_ID)",
    )

    pars.add_argument(
        "--run-fold",
        type=int,
        choices=range(-1, 5),
        metavar="[-1-4]",
        default=0,
        help="int value indicating which fold (in the range 0-4) to run",
    )

    pars.add_argument(
        "--run-validation-only",
        type=str2bool,
        default="no",
        help="Flag to run only the Validation step ( after the Training step is completed). Default ``no``.",
    )

    pars.add_argument(
        "--post-processing-folds",
        type=str,
        nargs="+",
        required=False,
        default="-1",
        help="Trained Folds to include in the post-processing and model export. Default ``-1`` (All Folds are used).",
    )

    pars.add_argument(
        "--output-model-file",
        type=str,
        required=False,
        default=None,
        help="File Path where to save the zipped Model File.",
    )

    pars.add_argument(
        "--resume-training",
        type=str2bool,
        default="no",
        help="Flag to indicate training resume after stopping it. Default ``no``.",
    )

    pars.add_argument(
        "--n-workers",
        type=str,
        default=None,
        help="Number of parallel processes used when pre-processing and unpacking the image data (Default: ``N_THREADS``)",
    )

    add_verbosity_options_to_argparser(pars)

    return pars


def main():
    parser = get_arg_parser()

    arguments, unknown_arguments = parser.parse_known_args()
    args = vars(arguments)

    logger = get_logger(  # NOQA: F841
        name=Path(__file__).name,
        level=log_lvl_from_verbosity_args(args),
    )

    config_file = args["config_file"]

    with open(config_file) as json_file:
        data = json.load(json_file)

        arguments = [
            "nnUNetv2_train",
            data["Task_ID"],
            "3d_fullres",
            str(args["run_fold"])
        ]
        if args["run_validation_only"]:
            arguments.append("--val")

        if args["resume_training"]:
            arguments.append("--c")

        arguments.extend(unknown_arguments)

        if not "N_THREADS" in os.environ:
            os.environ["N_THREADS"] = str(os.cpu_count())
        n_workers = "1"
        if args["n_workers"] is None:
            if "N_THREADS" in os.environ is not None:
                n_workers = str(os.environ["N_THREADS"])
        else:
            n_workers = str(args["n_workers"])

        os.environ["nnUNet_raw"] = str(Path(data["base_folder"]).joinpath("nnUNet_raw_data"))
        os.environ["nnUNet_preprocessed"] = data["preprocessing_folder"]
        os.environ["nnUNet_results"] = data["results_folder"]
        os.environ["nnUNet_def_n_proc"] = n_workers
        os.environ["nnUNet_n_proc_DA"] = n_workers

        if args["output_model_file"] is None:
            args["output_model_file"] = str(
                Path(data["results_folder"]).joinpath(data["Experiment Name"] + "_nnUNet_3d_fullres.zip"))

        if str(args["run_fold"]) == "-1" and "output_model_file" in args:
            if args["post_processing_folds"] != "-1":
                cmd = ["nnUNetv2_find_best_configuration", data["Task_ID"], "-c", "3d_fullres", "-f",
                       *args["post_processing_folds"]]
                cmd.extend(unknown_arguments)
                subprocess.run(cmd)
                cmd = ["nnUNetv2_export_model_to_zip", "-d", data["Task_ID"], "--exp_cv_preds", "-f",
                       *args["post_processing_folds"], "-c", "3d_fullres", "-o", args["output_model_file"]
                       ]
                cmd.extend(unknown_arguments)
                subprocess.run(cmd)
            else:
                cmd = ["nnUNetv2_find_best_configuration", data["Task_ID"], "-c", "3d_fullres"]
                cmd.extend(unknown_arguments)
                subprocess.run(cmd)
                cmd = ["nnUNetv2_export_model_to_zip", "-d", data["Task_ID"], "--exp_cv_preds", "-c", "3d_fullres",
                       "-o",
                       args["output_model_file"]]
                cmd.extend(unknown_arguments)
                subprocess.run(
                    cmd)
        else:
            subprocess.run(arguments)


if __name__ == "__main__":
    main()



#!/usr/bin/env python

import datetime
import subprocess
from argparse import ArgumentParser, RawTextHelpFormatter
from pathlib import Path
from textwrap import dedent

from PyMAIA.utils.log_utils import get_logger, add_verbosity_options_to_argparser, log_lvl_from_verbosity_args, INFO

TIMESTAMP = "{:%Y-%m-%d_%H-%M-%S}".format(datetime.datetime.now())

DESC = dedent(
    """
    Run pipeline steps from a TXT file.
    """  # noqa: E501
)
EPILOG = dedent(
    """
    Example call:
    ::
        {filename} --file PIPELINE_FILE.txt
    """.format(  # noqa: E501
        filename=Path(__file__).stem
    )
)


def get_arg_parser():
    pars = ArgumentParser(description=DESC, epilog=EPILOG, formatter_class=RawTextHelpFormatter)

    pars.add_argument(
        "--file",
        type=str,
        required=True,
        help="TXT file including list of commands to run.",
    )

    pars.add_argument(
        "--steps",
        type=str,
        nargs="+",
        required=False,
        help="Optional pipeline steps to run. When omitted, run all the steps.",
    )

    add_verbosity_options_to_argparser(pars)

    return pars


def main():
    parser = get_arg_parser()

    arguments = vars(parser.parse_args())

    logger = get_logger(  # noqa: F841
        name=Path(__file__).name,
        level=log_lvl_from_verbosity_args(arguments),
    )

    with open(arguments["file"]) as f:
        commands = f.readlines()

    steps = range(len(commands))
    if arguments["steps"] is not None:
        steps = arguments["steps"]
        steps = [int(step) for step in steps]

    for it, command in enumerate(commands):
        if it in steps:
            logger.log(INFO, "Running Step {}: {}".format(it, command))
            subprocess.call(command[:-1].split(" "))


if __name__ == "__main__":
    main()



#!/usr/bin/env python

import datetime
import json
import os
from argparse import ArgumentParser, RawTextHelpFormatter
from pathlib import Path
from textwrap import dedent

from PyMAIA.utils.file_utils import subfolders, subfiles
from PyMAIA.utils.log_utils import add_verbosity_options_to_argparser, str2bool
from PyMAIA.utils.seg_mask_utils import semantic_segmentation_to_instance

TIMESTAMP = "{:%Y-%m-%d_%H-%M-%S}".format(datetime.datetime.now())

DESC = dedent(
    """
    Script to convert a semantic segmentation dataset (with the `Patient ID` as the folder name) into an instance segmentation dataset.
    Instance segmentation masks are saved within the same patient folder with the standard format "INST_SEG.nii.gz". Regions in instance 
    segmentation containing less than 10 voxels are ignored and the number of labels in each instance segmentation mask is saved in a 
    separate json file ('inst_seg_labels.json') alongside its 'Patient ID'. 
    """  # noqa: E501
)
EPILOG = dedent(
    """
    Example call:
    ::
        {filename}  --data-folder /PATH/TO/SEMANTIC_SEG_DATA --sem-seg-suffix _SEG.nii.gz --inst-seg-suffix _INST_SEG.nii.gz --output-json-path /PATH/TO/JSON/inst_seg_labels.json
    """.format(  # noqa: E501
        filename=Path(__file__).stem
    )
)


def get_arg_parser():
    pars = ArgumentParser(description=DESC, epilog=EPILOG, formatter_class=RawTextHelpFormatter)

    pars.add_argument(
        "--data-folder",
        type=str,
        required=True,
        help="AutoPET patient dataset folder.",
    )

    pars.add_argument(
        "--sem-seg-suffix",
        type=str,
        required=True,
        help="Semantic Segmentation suffix.",
    )

    pars.add_argument(
        "--inst-seg-suffix",
        type=str,
        required=True,
        help="Instance Segmentation suffix.",
    )

    pars.add_argument(
        "--inst-seg-folder",
        type=str,
        required=False,
        help="Instance Segmentation Folder.Required only for the Decathlon-format Dataset.",
    )
    pars.add_argument(
        "--output-json-path",
        type=str,
        required=True,
        help="Output path of json file.",
    )

    pars.add_argument(
        "--decathlon-format",
        type=str2bool,
        required=False,
        default="no",
        help="Flag to specify if the Dataset is in the Decathlon format.",
    )

    add_verbosity_options_to_argparser(pars)

    return pars


def main():
    parser = get_arg_parser()
    arguments = vars(parser.parse_args())


    sem_seg = arguments["sem_seg_suffix"]
    inst_seg = arguments["inst_seg_suffix"]
    out_json = arguments["output_json_path"]

    # e.g. subject + sem_seg = "PETCT_0011f3deaf_0_SEG.nii.gz"
    labels_dict = {}

    if arguments["decathlon_format"]:
        subjects = subfiles(arguments["data_folder"], join=False)
        for subject in subjects:
            subject_id = subject[:-len(sem_seg)]
            subject_sem_seg_filename = os.path.join(arguments["data_folder"], subject)
            subject_inst_seg_filename = os.path.join(arguments["inst_seg_folder"], str(subject_id+sem_seg))
            num_features = semantic_segmentation_to_instance(subject_sem_seg_filename, subject_inst_seg_filename)
            labels_dict.update({str(subject_id): num_features})
            print("Subject: ", subject_id, " converted mask done.")
    else:
        subjects = subfolders(arguments["data_folder"], join=False)
        for subject in subjects:
            subject_sem_seg_filename = os.path.join(arguments["data_folder"], subject, str(subject + sem_seg))
            subject_inst_seg_filename = os.path.join(arguments["data_folder"], subject, str(subject + inst_seg))
            num_features = semantic_segmentation_to_instance(subject_sem_seg_filename, subject_inst_seg_filename)
            labels_dict.update({str(subject): num_features})
            print("Subject: ", subject, " converted mask done.")

    # Create Json file with number of labels of instance segmentation for each patient.
    with open(out_json, 'w') as json_file:
        json.dump(labels_dict, json_file)


if __name__ == "__main__":
    main()



#!/usr/bin/env python

import datetime
import json
import os
from argparse import ArgumentParser, RawTextHelpFormatter
from pathlib import Path
from textwrap import dedent

from PyMAIA.utils.file_utils import subfolders
from PyMAIA.utils.log_utils import add_verbosity_options_to_argparser
from PyMAIA.utils.volume_utils import resample_image

TIMESTAMP = "{:%Y-%m-%d_%H-%M-%S}".format(datetime.datetime.now())

DESC = dedent(
    """
    Script to downsample the modality of a given NIFTI dataset, using another modality as reference.
    """  # noqa: E501
)
EPILOG = dedent(
    """
    Example call:
    ::
        {filename}  --data-folder /PATH/TO/NIFTI_DATASET --input-modality _CT.nii.gz --ref-modality _PET.nii.gz --output-modality _downsampled_CT.nii.gz
    """.format(  # noqa: E501
        filename=Path(__file__).stem
    )
)


def get_arg_parser():
    pars = ArgumentParser(description=DESC, epilog=EPILOG, formatter_class=RawTextHelpFormatter)

    pars.add_argument(
        "--data-folder",
        type=str,
        required=True,
        help="Patient NIFTI dataset folder.",
    )

    pars.add_argument(
        "--input-modality",
        type=str,
        required=True,
        help="Input modality suffix.",
    )

    pars.add_argument(
        "--ref-modality",
        type=str,
        required=True,
        help="Reference modality suffix.",
    )

    pars.add_argument(
        "--output-modality",
        type=str,
        required=True,
        help="Output modality suffix.",
    )

    add_verbosity_options_to_argparser(pars)

    return pars


def main():
    parser = get_arg_parser()
    arguments = vars(parser.parse_args())

    subjects = subfolders(arguments["data_folder"], join=False)
    input_modality = arguments["input_modality"]
    ref_modality = arguments["ref_modality"]
    output_modality = arguments["output_modality"]

    for subject in subjects:
        input_filename = os.path.join(arguments["data_folder"], subject, str(subject + input_modality))
        ref_filename = os.path.join(arguments["data_folder"], subject, str(subject + ref_modality))
        output_filename = os.path.join(arguments["data_folder"], subject, str(subject + output_modality))
        resample_image(input_filename, ref_filename, output_filename)
        print("Subject: ", subject, " downsampling done.")

if __name__ == "__main__":
    main()



#!/usr/bin/env python

import datetime
import json
import os
from argparse import ArgumentParser, RawTextHelpFormatter
from multiprocessing import Pool
from pathlib import Path
from textwrap import dedent

from tqdm import tqdm

from PyMAIA.utils.file_utils import convert_nifti_pred_to_dicom_seg
from PyMAIA.utils.file_utils import subfolders
from PyMAIA.utils.log_utils import get_logger, add_verbosity_options_to_argparser, log_lvl_from_verbosity_args

if "N_THREADS" not in os.environ:
    os.environ["N_THREADS"] = "1"

TIMESTAMP = "{:%Y-%m-%d_%H-%M-%S}".format(datetime.datetime.now())

DESC = dedent(
    """
    Script to convert NIFTI Predictions ( as output from the model inference ), back into the original DICOM context ( as SEG modalities ).
    The NIFTI Predictions are converted into DICOM SEG and assigned to the corresponding original DICOM Study. 
    """  # noqa: E501
)
EPILOG = dedent(
    """
    Example call:
    ::
        {filename} --data-folder <NIFTI_DATA_FOLDER> --dicom-folder <DICOM_DATA_FOLDER> --output-folder <DICOM_SEG_FOLDER> --pred-suffix _seg.nii.gz --template-file <DCMQI_TEMPLATE> --study-id-summary STUDY_ID_DICT.json
    """.format(  # noqa: E501
        filename=Path(__file__).stem
    )
)


def get_arg_parser():
    pars = ArgumentParser(description=DESC, epilog=EPILOG, formatter_class=RawTextHelpFormatter)

    pars.add_argument(
        "--data-folder",
        type=str,
        required=True,
        help="Data Folder containing the NIFTI Predictions.",
    )

    pars.add_argument(
        "--dicom-folder",
        type=str,
        required=True,
        help="DICOM Folder containing the original Data.",
    )

    pars.add_argument(
        "--output-folder",
        type=str,
        required=True,
        help="Folder where to store the generated DICOM SEG.",
    )

    pars.add_argument(
        "--pred-suffix",
        type=str,
        required=True,
        help="Suffix to append to the Patient folder name to identify the NIFTI prediction.",
    )

    pars.add_argument(
        "--n-workers",
        type=str,
        required=False,
        default=os.environ["N_THREADS"],
        help="Number of Parallel Threads.",
    )

    pars.add_argument(
        "--template-file",
        type=str,
        required=True,
        help="DCMQI DICOM SEG Template, generated from http://qiicr.org/dcmqi/#/seg",
    )

    pars.add_argument(
        "--study-id-summary",
        type=str,
        required=True,
        help="Study ID Summary, generated when converting the DICOM Dataset to a NIFTI Dataset.",
    )

    add_verbosity_options_to_argparser(pars)

    return pars


def main():
    parser = get_arg_parser()

    arguments = vars(parser.parse_args())

    logger = get_logger(
        name=Path(__file__).name,
        level=log_lvl_from_verbosity_args(arguments),
    )

    with open(arguments["study_id_summary"], "r") as f:
        study_id_dict = json.load(f)

    subjects = subfolders(arguments["data_folder"], join=False)

    pool = Pool(int(arguments["n_workers"]))
    Path(arguments["output_folder"]).mkdir(parents=True, exist_ok=True)
    NIFTI_to_DICOM_conversions = []
    for subject in subjects:
        dicom_subject = subject[:-2]
        subject_dicom_folder = str(Path(arguments["dicom_folder"]).joinpath(dicom_subject))
        subject_nifti_pred = str(Path(arguments["data_folder"]).joinpath(subject, subject + arguments["pred_suffix"]))

        NIFTI_to_DICOM_conversions.append(
            pool.starmap_async(
                convert_nifti_pred_to_dicom_seg,
                ((
                     subject_nifti_pred,
                     subject_dicom_folder,
                     arguments["template_file"],
                     str(Path(arguments["output_folder"]).joinpath(subject + ".dcm")),
                     study_id_dict[dicom_subject][subject[-1]]

                 ),),
            )
        )

    patients_map = {}
    for i in tqdm(NIFTI_to_DICOM_conversions):
        _ = i.get()


if __name__ == "__main__":
    main()



#!/usr/bin/env python

import datetime
import json
import os
from argparse import ArgumentParser, RawTextHelpFormatter
from multiprocessing import Pool
from pathlib import Path
from textwrap import dedent

from tqdm import tqdm

from PyMAIA.utils.file_utils import subfolders
from PyMAIA.utils.log_utils import add_verbosity_options_to_argparser, get_logger, log_lvl_from_verbosity_args
from PyMAIA.utils.volume_utils import convert_DICOM_folder_to_NIFTI_image

TIMESTAMP = "{:%Y-%m-%d_%H-%M-%S}".format(datetime.datetime.now())

DESC = dedent(
    """
    Script to convert a ``DICOM`` dataset (structured as Patient-Study-Series) into a NIFTI format (with the `Patient ID` as the folder name).
    When multiple studies for the same patient are found, different **DICOM studies** are saved in different folders, appending the study index to the patient name.
    *DICOM series* for the same study are saved in the same patient folder.
    """  # noqa: E501
)
EPILOG = dedent(
    """
    Example call:
    ::
        {filename}  --data-folder /PATH/TO/DICOM_DATA --output-folder /PATH/TO/NIFTI_DATASET
    """.format(  # noqa: E501
        filename=Path(__file__).stem
    )
)

if "N_THREADS" not in os.environ:
    os.environ["N_THREADS"] = "1"


def get_arg_parser():
    pars = ArgumentParser(description=DESC, epilog=EPILOG, formatter_class=RawTextHelpFormatter)

    pars.add_argument(
        "--data-folder",
        type=str,
        required=True,
        help="DICOM Dataset folder.",
    )

    pars.add_argument(
        "--output-folder",
        type=str,
        required=True,
        help="Output folder where to save the converted NIFTI dataset.",
    )

    pars.add_argument(
        "--n-workers",
        type=int,
        required=False,
        default=os.environ["N_THREADS"],
        help="Number of worker threads to use. (Default: {})".format(os.environ["N_THREADS"]),
    )

    add_verbosity_options_to_argparser(pars)

    return pars


def main():
    parser = get_arg_parser()
    arguments = vars(parser.parse_args())

    logger = get_logger(  # NOQA: F841
        name=Path(__file__).name,
        level=log_lvl_from_verbosity_args(arguments),
    )
    subjects = subfolders(arguments["data_folder"], join=False)

    pool = Pool(int(arguments["n_workers"]))
    Path(arguments["output_folder"]).mkdir(parents=True, exist_ok=True)
    DICOM_to_NIFTI_conversions = []
    for subject in subjects:
        subject_dicom_folder = str(Path(arguments["data_folder"]).joinpath(subject))
        subject_nifti_filename = str(Path(arguments["output_folder"]).joinpath(subject))
        DICOM_to_NIFTI_conversions.append(
            pool.starmap_async(
                convert_DICOM_folder_to_NIFTI_image,
                ((subject_dicom_folder, subject_nifti_filename),),
            )
        )

    patients_map = {}
    for i in tqdm(DICOM_to_NIFTI_conversions):
        patient_study_map = i.get()
        patients_map[list(patient_study_map[0].keys())[0]] = patient_study_map[0][list(patient_study_map[0].keys())[0]]

    with open(Path(arguments["output_folder"]).parent.joinpath(Path(arguments["output_folder"]).name + ".json"),
              "w") as file:
        json.dump(patients_map, file)


if __name__ == "__main__":
    main()



#!/usr/bin/env python

import json
import os
from argparse import ArgumentParser, RawTextHelpFormatter
from pathlib import Path
from textwrap import dedent

from PyMAIA.utils.log_utils import (
    get_logger,
    add_verbosity_options_to_argparser,
    log_lvl_from_verbosity_args,
)

DESC = dedent(
    """
    Run nnUNet command to extract dataset fingerprints, create experiment plan, preprocessing and ( optionally ) verify 
    the dataset integrity.
    The CL script called is  ``nnUNetv2_plan_and_preprocess``, with the arguments extracted from the given configuration file.
    """  # noqa: E501
)
EPILOG = dedent(
    """
    Example call:
    ::
        {filename} --config-file /PATH/TO/CONFIG_FILE.json
    """.format(  # noqa: E501
        filename=Path(__file__).name
    )
)


def get_arg_parser():
    pars = ArgumentParser(description=DESC, epilog=EPILOG, formatter_class=RawTextHelpFormatter)

    pars.add_argument(
        "--config-file",
        type=str,
        required=True,
        help="File path for the configuration dictionary, used to retrieve experiments variables (Task_ID) ",
    )

    pars.add_argument(
        "--n-workers",
        type=str,
        default=None,
        help="Number of parallel processes used when pre-processing and unpacking the image data (Default: ``N_THREADS``)",
    )

    add_verbosity_options_to_argparser(pars)

    return pars


def main():
    parser = get_arg_parser()
    arguments, unknown_arguments = parser.parse_known_args()
    args = vars(arguments)

    logger = get_logger(  # NOQA: F841
        name=Path(__file__).name,
        level=log_lvl_from_verbosity_args(args),
    )
    config_file = args["config_file"]

    if not "N_THREADS" in os.environ:
        os.environ["N_THREADS"] = str(os.cpu_count())
    n_workers = "1"
    if args["n_workers"] is None:
        if "N_THREADS" in os.environ is not None:
            n_workers = str(os.environ["N_THREADS"])
    else:
        n_workers = str(args["n_workers"])

    with open(config_file) as json_file:
        data = json.load(json_file)

        arguments = [
            "-d",
            data["Task_ID"],
            "-c",
            "3d_fullres",
            "-np",
            n_workers,
            "-npfp",
            n_workers
        ]

        os.environ["nnUNet_raw"] = str(Path(data["base_folder"]).joinpath("nnUNet_raw_data"))
        os.environ["nnUNet_preprocessed"] = data["preprocessing_folder"]
        os.environ["nnUNet_def_n_proc"] = n_workers
        os.environ["nnUNet_results"] = data["results_folder"]
        arguments.extend(unknown_arguments)
        os.system("nnUNetv2_plan_and_preprocess " + " ".join(arguments))


if __name__ == "__main__":
    main()



#!/usr/bin/env python

import json
import os
from argparse import ArgumentParser, RawTextHelpFormatter
from pathlib import Path
from textwrap import dedent

from PyMAIA.utils.log_utils import (
    get_logger,
    add_verbosity_options_to_argparser,
    log_lvl_from_verbosity_args,
)

DESC = dedent(
    """
    Run nnDetection pre-processing, data analysis, and image data unpacking.
    The CL PyMAIA_scripts called are  ``nndet_prep`` and ``nndet_unpack``, with the arguments extracted from the given configuration file.
    """  # noqa: E501
)
EPILOG = dedent(
    """
    Example call:
    ::
        {filename} --config-file /PATH/TO/CONFIG_FILE.json
    """.format(  # noqa: E501
        filename=Path(__file__).stem
    )
)


def get_arg_parser():
    pars = ArgumentParser(description=DESC, epilog=EPILOG, formatter_class=RawTextHelpFormatter)

    pars.add_argument(
        "--config-file",
        type=str,
        required=True,
        help="File path for the configuration dictionary, used to retrieve experiment variables (**Task_ID**, **results_folder**, ...) ",
        # noqa: E501
    )

    pars.add_argument(
        "--n-workers",
        type=str,
        default=None,
        help="Number of parallel processes used when pre-processing and unpacking the image data (Default: ``N_THREADS``)",
    )

    add_verbosity_options_to_argparser(pars)

    return pars


def main():
    parser = get_arg_parser()
    arguments, unknown_arguments = parser.parse_known_args()
    args = vars(arguments)

    logger = get_logger(  # NOQA: F841
        name=Path(__file__).name,
        level=log_lvl_from_verbosity_args(args),
    )
    config_file = args["config_file"]

    if not "N_THREADS" in os.environ:
        os.environ["N_THREADS"] = str(os.cpu_count())

    n_workers = "1"
    if args["n_workers"] is None:
        if "N_THREADS" in os.environ is not None:
            n_workers = str(os.environ["N_THREADS"])
    else:
        n_workers = str(args["n_workers"])

    with open(config_file) as json_file:
        data = json.load(json_file)

        arguments = [data["Task_ID"], "-np", n_workers, "-npp", n_workers]

        os.environ["det_data"] = data["base_folder"]
        os.environ["OMP_NUM_THREADS"] = "1"

        os.environ["nnUNet_def_n_proc"] = n_workers
        os.environ["det_models"] = data["results_folder"]
        arguments.extend(unknown_arguments)
        os.system("nndet_prep " + " ".join(arguments))
        os.system(
            "nndet_unpack {} {}".format(
                str(
                    Path(os.environ["det_data"]).joinpath(
                        "Task{}_{}".format(data["Task_ID"], data["Task_Name"]), "preprocessed", "D3V001_3d", "imagesTr"
                    )
                ),
                n_workers,
            )
        )


if __name__ == "__main__":
    main()



import nibabel as nib
from scipy.ndimage import label, generate_binary_structure


def semantic_segmentation_to_instance(mask_filename: str, output_path: str) -> int:
    """
    Given a semantic segmentation mask convert to instance segmentation and save in the given output path.
    Return the number of labels in instance segmentation mask.

    Parameters
    ----------
    mask_filename:
        File path of semantic segmentation mask.
    output_path:
        Output path including new instance segmentation mask file name.

    Returns
    -------
        Number of labels in converted instance segmentation mask.
    """

    # load segmentation mask and properties
    mask = nib.load(mask_filename)
    affine = mask.affine
    np_mask = mask.get_fdata()

    # label connected regions in segmentation mask
    labeled_array, num_features = label(np_mask, structure=generate_binary_structure(3, 3))

    thresh = 10
    # voxel count in each region from https://neurostars.org/t/roi-voxel-count-using-python/6451
    # ignore regions below threshold = 10
    for i in range(1, num_features + 1):
        vox_count = (labeled_array == i).sum()
        if vox_count < thresh:
            labeled_array[labeled_array == i] = 0

    labeled_array[labeled_array > 0] = 1
    labeled_array, num_features = label(labeled_array, structure=generate_binary_structure(3, 3))

    # convert labeled array into Nifti file
    labeled_mask = nib.Nifti1Image(labeled_array, affine=affine)
    nib.save(labeled_mask, output_path)
    return num_features



#!/usr/bin/env python

import json
from argparse import ArgumentParser, RawTextHelpFormatter
from distutils.dir_util import copy_tree
from pathlib import Path
from textwrap import dedent

# from old_src.evaluation import get_results_summary_filepath
from PyMAIA.utils.file_utils import subfolders, order_data_folder_by_patient
from PyMAIA.utils.log_utils import get_logger, add_verbosity_options_to_argparser, log_lvl_from_verbosity_args

DESC = dedent(
    """
    Script used to copy and save nnDetection Experiment predictions, from the original experiment folder to the specified output folder.
    """  # noqa: E501 W291 W605
)
EPILOG = dedent(
    """
    Example call:
    ::
        {filename} --config-file /path/to/config_file.json --output-experiment-folder /home/Experiment_Predictions
    """.format(  # noqa: E501 W291
        filename=Path(__file__).name
    )
)


def get_arg_parser():
    pars = ArgumentParser(description=DESC, epilog=EPILOG, formatter_class=RawTextHelpFormatter)

    pars.add_argument(
        "--config-file",
        type=str,
        required=True,
        help="File path for the configuration dictionary, used to retrieve experiment settings ",
    )

    pars.add_argument(
        "--output-experiment-folder",
        type=str,
        required=True,
        help="Folder path to set the output experiment folder.",
    )

    add_verbosity_options_to_argparser(pars)
    return pars


def main():
    parser = get_arg_parser()

    args = vars(parser.parse_args())

    logger = get_logger(  # NOQA: F841
        name=Path(__file__).name,
        level=log_lvl_from_verbosity_args(args),
    )

    with open(args["config_file"]) as json_file:
        config_dict = json.load(json_file)

    output_path = args["output_experiment_folder"]

    sections = ["validation"]

    for section in sections:
        prediction_directory_out = Path(output_path).joinpath(
            "Task" + config_dict["Task_ID"] + "_{}".format(config_dict["Task_Name"]), section
        )
        prediction_directory_out.mkdir(parents=True, exist_ok=True)
        prediction_directory = Path(config_dict["results_folder"]).joinpath(
            "Task" + config_dict["Task_ID"] + "_{}".format(config_dict["Task_Name"]),
            "RetinaUNetV001_D3V001_3d",
            "consolidated",
            "val_predictions_nii",
        )
        if prediction_directory.is_dir():
            order_data_folder_by_patient(prediction_directory, "_boxes.json")
            subject_predictions = subfolders(prediction_directory, join=False)
            for subject_prediction in subject_predictions:
                copy_tree(
                    str(Path(prediction_directory).joinpath(subject_prediction)),
                    str(Path(prediction_directory_out).joinpath(subject_prediction)),
                )



if __name__ == "__main__":
    main()



import os

import torch

from PyMAIA.utils.log_utils import get_logger

try:

    GPU_AVAILABLE = torch.cuda.is_available()


except:  # noqa: E722
    GPU_AVAILABLE = False

logger = get_logger("PyMAIA", "INFO")
logger.info("PyMAIA GPU Available: {}".format(GPU_AVAILABLE))
os.environ["GPU_AVAILABLE"] = str(GPU_AVAILABLE)  # noqa: F405

from . import _version

__version__ = _version.get_versions()['version']



#!/usr/bin/env python

import csv
import datetime
import importlib.resources
import json
import os
from argparse import ArgumentParser, RawTextHelpFormatter
from pathlib import Path
from textwrap import dedent

import numpy as np
from sklearn.model_selection import KFold

import PyMAIA.configs
from PyMAIA.utils.file_utils import (
    create_nnunet_data_folder_tree,
    split_dataset,
    copy_data_to_dataset_folder,
    copy_data_from_dict_to_dataset_folder,
    save_config_json,
    generate_dataset_json,
)
from PyMAIA.utils.log_utils import get_logger, add_verbosity_options_to_argparser, log_lvl_from_verbosity_args

TIMESTAMP = "{:%Y-%m-%d_%H-%M-%S}".format(datetime.datetime.now())

DESC = dedent(
    """
    Prepare Dataset folder according to the nnUNet specifications, creating and populating the subfolders ``imagesTr``,
    ``labelsTr``, ``imagesTs`` and ``labelsTs``. In addition, a JSON instance configuration file (as required by nnUNet)
    for each label mask is generated, alongside a summary of the train/test split of the dataset.
    """  # noqa: E501
)
EPILOG = dedent(
    """
    Example call:
    ::
        {filename} -i /PATH/TO/DATA_FOLDER --task-ID 000 --task-name Example --config-file Example_config.json
        {filename} -i /PATH/TO/DATA_FOLDER --task-ID 000 --task-name Example --config-file Example_config.json --test-split 30
    """.format(  # noqa: E501
        filename=Path(__file__).name
    )
)


def main():
    parser = get_arg_parser()

    arguments = vars(parser.parse_args())

    logger = get_logger(
        name=Path(__file__).name,
        level=log_lvl_from_verbosity_args(arguments),
    )

    try:
        with open(arguments["config_file"]) as json_file:
            config_dict = json.load(json_file)
    except FileNotFoundError:
        with importlib.resources.path(PyMAIA.configs, arguments["config_file"]) as json_path:
            with open(json_path) as json_file:
                config_dict = json.load(json_file)

    os.environ["raw_data_base"] = str(
        Path(os.environ["ROOT_FOLDER"]).joinpath(
            config_dict["Experiment Name"], config_dict["Experiment Name"] + "_base"
        )
    )
    os.environ["preprocessed_folder"] = str(
        Path(os.environ["ROOT_FOLDER"]).joinpath(
            config_dict["Experiment Name"], config_dict["Experiment Name"] + "_preprocess"
        )
    )
    os.environ["RESULTS_FOLDER"] = str(
        Path(os.environ["ROOT_FOLDER"]).joinpath(
            config_dict["Experiment Name"], config_dict["Experiment Name"] + "_results"
        )
    )
    try:
        dataset_path = str(
            Path(os.environ["raw_data_base"]).joinpath(
                "nnUNet_raw_data",
                "Dataset" + arguments["task_ID"] + "_" + arguments["task_name"],
            )
        )

    except KeyError:
        logger.error("raw_data_base is not set as environment variable")
        return 1

    create_nnunet_data_folder_tree(
        os.environ["raw_data_base"],
        arguments["task_name"],
        arguments["task_ID"],
    )

    if Path(arguments["input_data_folder"]).is_file() and arguments["input_data_folder"].endswith(".json"):
        with open(arguments["input_data_folder"]) as json_file:
            dataset_dict = json.load(json_file)
            train_dataset = dataset_dict["train"]
            train_dataset = [Path(train_id[list(config_dict["Modalities"].values())[0]]).name[
                             :-len(list(config_dict["Modalities"].keys())[0])] for train_id in train_dataset]
            if "test" in dataset_dict:
                test_dataset = dataset_dict["test"]
                test_dataset = [Path(test_id[list(config_dict["Modalities"].values())[0]]).name[
                                :-len(list(config_dict["Modalities"].keys())[0])] for test_id in test_dataset]

            else:
                test_dataset = []
    else:
        train_dataset, test_dataset = split_dataset(arguments["input_data_folder"], arguments["test_split"],
                                                    config_dict["Seed"])

    dataset_split = []
    for test_subject in test_dataset:
        dataset_split_dict = {"Subject": test_subject, "Split": "Testing"}
        dataset_split.append(dataset_split_dict)

    train_dataset_sorted = np.sort(train_dataset)
    kfold = KFold(n_splits=config_dict["n_folds"], shuffle=True, random_state=config_dict["Seed"])
    for i, (train_idx, test_idx) in enumerate(kfold.split(train_dataset_sorted)):
        for test in test_idx:
            dataset_split_dict = {"Subject": train_dataset_sorted[test], "Split": "Validation_fold_{}".format(i)}
            dataset_split.append(dataset_split_dict)

    dataset_split_summary = Path(os.environ["ROOT_FOLDER"]).joinpath(
        config_dict["Experiment Name"], "dataset_split.csv"
    )

    with open(dataset_split_summary, "w") as csvfile:
        writer = csv.DictWriter(csvfile, fieldnames=["Subject", "Split"])
        writer.writeheader()
        for data in dataset_split:
            writer.writerow(data)

    if Path(arguments["input_data_folder"]).is_file() and arguments["input_data_folder"].endswith(".json"):
        config_dict["dataset_folder"] = str(Path(arguments["input_data_folder"]))
    else:
        config_dict["dataset_folder"] = str(Path(arguments["input_data_folder"]))

    if Path(arguments["input_data_folder"]).is_file() and arguments["input_data_folder"].endswith(".json"):
        copy_data_from_dict_to_dataset_folder(
            arguments["input_data_folder"],
            "train",
            Path(dataset_path).joinpath("imagesTr"),
            config_dict,
            Path(dataset_path).joinpath("labelsTr"),

        )
        copy_data_from_dict_to_dataset_folder(
            arguments["input_data_folder"],
            "test",
            Path(dataset_path).joinpath("imagesTs"),
            config_dict,
            Path(dataset_path).joinpath("labelsTs"),
        )
    else:
        copy_data_to_dataset_folder(
            arguments["input_data_folder"],
            train_dataset,
            Path(dataset_path).joinpath("imagesTr"),
            config_dict,
            Path(dataset_path).joinpath("labelsTr")
        )
        copy_data_to_dataset_folder(
            arguments["input_data_folder"],
            test_dataset,
            Path(dataset_path).joinpath("imagesTs"),
            config_dict,
            Path(dataset_path).joinpath("labelsTs")
        )

    if "RegionClassOrder" in config_dict:
        region_class_order = config_dict["RegionClassOrder"]
    else:
        region_class_order = None
    generate_dataset_json(
        str(
            Path(dataset_path).joinpath("dataset.json"
                                        )
        ),
        train_dataset,
        test_dataset,
        list(config_dict["Modalities"].values()),
        config_dict["label_dict"],
        task_name="Dataset{}_{}".format(arguments["task_ID"], arguments["task_name"]),
        file_extension=config_dict["FileExtension"],
        nnunet_format=True,
        region_class_order=region_class_order

    )

    config_dict["Task_ID"] = arguments["task_ID"]
    config_dict["Task_Name"] = arguments["task_name"]
    config_dict["train_test_split"] = arguments["test_split"]
    config_dict["base_folder"] = os.environ["raw_data_base"]

    output_json_basename = "Dataset" + arguments["task_ID"] + "_" + config_dict["Experiment Name"] + ".json"

    try:
        config_dict["results_folder"] = os.environ["RESULTS_FOLDER"]
        Path(config_dict["results_folder"]).mkdir(parents=True, exist_ok=True)
    except KeyError:
        logger.warning(
            "RESULTS_FOLDER is not set as environment variable, {} is not saved".format(output_json_basename))
        return 1
    try:
        config_dict["preprocessing_folder"] = os.environ["preprocessed_folder"]
        Path(config_dict["preprocessing_folder"]).mkdir(parents=True, exist_ok=True)

    except KeyError:
        logger.warning(
            "preprocessed_folder is not set as environment variable, not saved in {}".format(output_json_basename)
            # noqa E501
        )
    save_config_json(config_dict, str(Path(config_dict["results_folder"]).joinpath(output_json_basename)))


def get_arg_parser():
    pars = ArgumentParser(description=DESC, epilog=EPILOG, formatter_class=RawTextHelpFormatter)

    pars.add_argument(
        "-i",
        "--input-data-folder",
        type=str,
        required=True,
        help="Input Dataset folder",
    )

    pars.add_argument(
        "--task-ID",
        type=str,
        default="100",
        help="Task ID used in the folder path tree creation (Default: 100)",
    )

    pars.add_argument(
        "--task-name",
        type=str,
        required=True,
        help="Task Name used in the folder path tree creation.",  # noqa E501
    )

    pars.add_argument(
        "--test-split",
        type=int,
        choices=range(0, 101),
        metavar="[0-100]",
        default=20,
        help="Split value ( in %% ) to create Test set from Dataset (Default: 20)",
    )

    pars.add_argument(
        "--config-file",
        type=str,
        required=True,
        help="Configuration JSON file with experiment and dataset parameters.",
    )

    add_verbosity_options_to_argparser(pars)

    return pars


if __name__ == "__main__":
    main()



#!/usr/bin/env python

import json
import pickle
from argparse import ArgumentParser, RawTextHelpFormatter
from pathlib import Path
from textwrap import dedent
from typing import Dict, List, Sequence

import numpy as np
import pandas as pd
from sklearn.model_selection import KFold

from PyMAIA.utils.log_utils import get_logger, add_verbosity_options_to_argparser, log_lvl_from_verbosity_args

DESC = dedent(
    """
    Script to perform Object Detection (COCO and FROC Metrics) and Segmentation (Dice score) evaluation on nnDetection experiments.
    
    By specifying  the ``class-file`` and ``classes`` parameters, the class-wise analysis of the metrics is performed.
    
    An Excel Spreadsheet and several PNG plots (representing FROC curves and Histogram Analysis), are produced as output.
    """  # noqa: E501
)
EPILOG = dedent(
    """
    Example call:
    ::
        {filename} --config-file /PATH/TO/CONFIG_FILE.json --output-dir /OUTPUT/PATH
        {filename} --config-file /PATH/TO/CONFIG_FILE.json --output-dir /OUTPUT/PATH --n-fold 0
    """.format(  # noqa: E501
        filename=Path(__file__).stem
    )
)


def iou_filter(image_dict: Dict[int, Dict[str, np.ndarray]], iou_idx: List[int],
               filter_keys: Sequence[str] = ('dtMatches', 'gtMatches', 'dtIgnore')):
    """
    This functions can be used to filter specific IoU values from the results
    to make sure that the correct IoUs are passed to metric

    Parameters
    ----------
    image_dict : dict
        dictionary containin :param:`filter_keys` which contains IoUs in the first dimension
    iou_idx : List[int]
        indices of IoU values to filter from keys
    filter_keys : tuple, optional
        keys to filter, by default ('dtMatches', 'gtMatches', 'dtIgnore')

    Returns
    -------
    dict
        filtered dictionary
    """
    iou_idx = list(iou_idx)
    filtered = {}
    for cls_key, cls_item in image_dict.items():
        filtered[cls_key] = {key: item[iou_idx] if key in filter_keys else item
                             for key, item in cls_item.items()}
    return filtered


def get_unique_iou_thresholds(metric):
    """
    Compute unique set of iou thresholds
    """
    iou_thresholds = [_i for _i in metric.get_iou_thresholds()]
    iou_thresholds = list(set(iou_thresholds))
    iou_thresholds.sort()
    return iou_thresholds


def get_indices_of_iou(metric):
    """
    Find indices of iou thresholds for each metric
    """

    return [get_unique_iou_thresholds(metric).index(th) for th in metric.get_iou_thresholds()]


def get_arg_parser():
    pars = ArgumentParser(description=DESC, epilog=EPILOG, formatter_class=RawTextHelpFormatter)

    pars.add_argument(
        "--config-file",
        type=str,
        required=True,
        help="File path for the configuration dictionary, used to retrieve experiments variables.",
    )

    pars.add_argument(
        "--run-fold",
        type=str,
        default="-1",
        required=False,
        help="Index indicating which fold to run. Default: ``-1``. If set to ``-1``, runs the metric evaluation on the "
             "consolidated predictions.",
    )

    pars.add_argument(
        "--output-dir",
        type=str,
        required=True,
        help="Folder path where to save the Excel spreadsheet and the PNG plots containing the evaluation metrics.",
    )

    pars.add_argument(
        "--class-file",
        type=str,
        required=False,
        help="Optional JSON file, containing a dictionary where each Subject ID is stored with the corresponding "
             "Subject Class."
             "If not specified, all the subjects will be analysed.",
    )

    pars.add_argument(
        "--classes",
        type=str,
        nargs="+",
        required=False,
        help="Optional list of Subject Classes, to be considered for the class-wise result analysis.",
    )

    pars.add_argument(
        "--n-folds",
        type=str,
        default="1",
        required=False,
        help="Number of Folds used to aggregate subjects and create Object Detection metrics statistic.",
    )

    pars.add_argument(
        "--model",
        type=str,
        default="RetinaUNetV001_D3V001_3d",
        required=False,
        help="nnDetection Model used for sweeping and consolidation. Default: ``RetinaUNetV001_D3V001_3d``",
    )

    add_verbosity_options_to_argparser(pars)

    return pars


def main():
    from nndet.evaluator.detection.coco import COCOMetric
    from nndet.evaluator.detection.froc import FROCMetric
    from nndet.evaluator.detection.hist import PredictionHistogram
    parser = get_arg_parser()

    arguments, unknown_arguments = parser.parse_known_args()
    args = vars(arguments)

    logger = get_logger(  # NOQA: F841
        name=Path(__file__).name,
        level=log_lvl_from_verbosity_args(args),
    )

    config_file = args["config_file"]

    with open(config_file) as json_file:
        data = json.load(json_file)

    output_dir = args["output_dir"]

    Path(output_dir).mkdir(parents=True, exist_ok=True)

    patients_classes_dict = None
    if args["class_file"] is not None and args["classes"] is not None:
        class_file = args["class_file"]
        patient_classes = args["classes"]
        with open(class_file, "rb") as file:
            patients_classes_dict = json.load(file)
    else:
        patient_classes = None

    if args["run_fold"] != "-1":
        results_folder = Path(data["results_folder"]).joinpath("Task{}_{}".format(data["Task_ID"], data["Task_Name"]),
                                                               args["model"],
                                                               "fold{}".format(args["run_fold"]),
                                                               "val_results"
                                                               )

    else:
        results_folder = Path(data["results_folder"]).joinpath(
            "Task{}_{}".format(data["Task_ID"], data["Task_Name"]),
            args["model"],
            "consolidated",
            "val_results"
        )

    boxes_metrics_file = str(Path(results_folder).joinpath("results_boxes_per_case.pkl"))

    boxes_ids_file = str(Path(results_folder).joinpath("results_boxes_per_case_IDs.json"))

    seg_file = str(Path(results_folder).joinpath("results_seg_per_case.json"))

    with open(seg_file, "rb") as file:
        patients_seg_dict = json.load(file)

    df_seg = []
    for patient in patients_seg_dict:
        if patients_classes_dict is not None:
            df_seg.append({"Subject": patient, "Metric": "Dice", "Score": patients_seg_dict[patient],
                           "Class": patients_classes_dict[patient]})
        else:
            df_seg.append({"Subject": patient, "Metric": "Dice", "Score": patients_seg_dict[patient]})

    df_seg = pd.DataFrame.from_records(df_seg)

    with open(boxes_metrics_file, "rb") as f:
        boxes_metrics = pickle.load(f)
    with open(boxes_ids_file, "rb") as file:
        boxes_ids = json.load(file)

    iou_thresholds = np.arange(0.1, 1.0, 0.1)
    iou_range = (0.1, 0.5, 0.05)
    per_class = True
    verbose = False
    classes = [label for label in data["label_dict"]]

    coco = COCOMetric(classes,
                      iou_list=iou_thresholds,
                      iou_range=iou_range,
                      max_detection=(100,),
                      per_class=per_class,
                      verbose=verbose,
                      )

    writer = pd.ExcelWriter(
        Path(output_dir).joinpath("{}.xlsx".format("Task{}_{}".format(data["Task_ID"], data["Task_Name"]))),
        engine='openpyxl')

    if patient_classes is None:
        froc = FROCMetric(classes,
                          iou_thresholds=iou_thresholds,
                          fpi_thresholds=(1 / 8, 1 / 4, 1 / 2, 1, 2, 4, 8),
                          per_class=per_class,
                          verbose=verbose,
                          save_dir=Path(output_dir)
                          )

        histo = PredictionHistogram(classes=classes,
                                    save_dir=Path(output_dir),
                                    iou_thresholds=(0.1, 0.5),
                                    )

        Path(output_dir).mkdir(parents=True, exist_ok=True)
        filtered_boxes_metrics = []
        if int(args["n_folds"]) == 1:
            df = []
            filtered_boxes_metrics = boxes_metrics
            iou_filtered_results = [iou_filter(boxes_metric, iou_idx=get_indices_of_iou(froc)) for boxes_metric in
                                    filtered_boxes_metrics]
            scores, curves = froc.compute(iou_filtered_results)

            for score in scores:
                df.append({"Metric": score, "Score": scores[score], "Group": 0})
            iou_filtered_results = [iou_filter(boxes_metric, iou_idx=get_indices_of_iou(coco)) for boxes_metric in
                                    filtered_boxes_metrics]
            scores, _ = coco.compute(iou_filtered_results)
            for score in scores:
                df.append({"Metric": score, "Score": scores[score], "Group": 0})
        else:
            kf = KFold(n_splits=int(args["n_folds"]), random_state=1234567, shuffle=True)
            df = []
            for i, (_, indexes) in enumerate(kf.split(list(range(len(boxes_metrics))))):
                filtered_boxes_metrics = []
                for idx in indexes:
                    filtered_boxes_metrics.append(boxes_metrics[idx])

                iou_filtered_results = [iou_filter(boxes_metric, iou_idx=get_indices_of_iou(froc)) for boxes_metric in
                                        filtered_boxes_metrics]
                scores, curves = froc.compute(iou_filtered_results)

                for score in scores:
                    df.append({"Metric": score, "Score": scores[score], "Group": i})
                iou_filtered_results = [iou_filter(boxes_metric, iou_idx=get_indices_of_iou(coco)) for boxes_metric in
                                        filtered_boxes_metrics]
                scores, _ = coco.compute(iou_filtered_results)
                for score in scores:
                    df.append({"Metric": score, "Score": scores[score], "Group": i})

        df = pd.DataFrame.from_records(df)

        df.to_excel(writer, sheet_name="Object Detection")

        iou_filtered_results = [iou_filter(boxes_metric, iou_idx=get_indices_of_iou(histo)) for boxes_metric in
                                filtered_boxes_metrics]
        _, _ = histo.compute(iou_filtered_results)
    else:
        for class_name in patient_classes:
            if int(args["n_folds"]) == 1:
                froc = FROCMetric(classes,
                                  iou_thresholds=iou_thresholds,
                                  fpi_thresholds=(1 / 8, 1 / 4, 1 / 2, 1, 2, 4, 8),
                                  per_class=per_class,
                                  verbose=verbose,
                                  save_dir=Path(output_dir).joinpath(class_name)
                                  )

                histo = PredictionHistogram(classes=classes,
                                            save_dir=Path(output_dir).joinpath(class_name),
                                            iou_thresholds=(0.1, 0.5),
                                            )

                Path(output_dir).joinpath(class_name).mkdir(parents=True, exist_ok=True)
                filtered_boxes_metrics = []
                for boxes_metric, id in zip(boxes_metrics, boxes_ids):
                    if patients_classes_dict[id] == class_name:
                        filtered_boxes_metrics.append(boxes_metric)
                df = []

                fold_filtered_boxes_metrics = filtered_boxes_metrics

                iou_filtered_results = [iou_filter(boxes_metric, iou_idx=get_indices_of_iou(froc)) for boxes_metric
                                        in
                                        fold_filtered_boxes_metrics]
                scores, curves = froc.compute(iou_filtered_results)

                for score in scores:
                    df.append({"Metric": score, "Score": scores[score], "Group": 0})

                iou_filtered_results = [iou_filter(boxes_metric, iou_idx=get_indices_of_iou(coco)) for boxes_metric
                                        in
                                        fold_filtered_boxes_metrics]
                scores, _ = coco.compute(iou_filtered_results)
                for score in scores:
                    df.append({"Metric": score, "Score": scores[score], "Group": 0})
            else:
                kf = KFold(n_splits=int(args["n_folds"]), random_state=1234567, shuffle=True)
                froc = FROCMetric(classes,
                                  iou_thresholds=iou_thresholds,
                                  fpi_thresholds=(1 / 8, 1 / 4, 1 / 2, 1, 2, 4, 8),
                                  per_class=per_class,
                                  verbose=verbose,
                                  save_dir=Path(output_dir).joinpath(class_name)
                                  )

                histo = PredictionHistogram(classes=classes,
                                            save_dir=Path(output_dir).joinpath(class_name),
                                            iou_thresholds=(0.1, 0.5),
                                            )

                Path(output_dir).joinpath(class_name).mkdir(parents=True, exist_ok=True)

                filtered_boxes_metrics = []
                for boxes_metric, id in zip(boxes_metrics, boxes_ids):
                    if patients_classes_dict[id] == class_name:
                        filtered_boxes_metrics.append(boxes_metric)
                df = []
                for i, (_, indexes) in enumerate(kf.split(list(range(len(filtered_boxes_metrics))))):
                    fold_filtered_boxes_metrics = []
                    for idx in indexes:
                        fold_filtered_boxes_metrics.append(filtered_boxes_metrics[idx])

                    iou_filtered_results = [iou_filter(boxes_metric, iou_idx=get_indices_of_iou(froc)) for boxes_metric
                                            in
                                            fold_filtered_boxes_metrics]
                    scores, curves = froc.compute(iou_filtered_results)

                    for score in scores:
                        df.append({"Metric": score, "Score": scores[score], "Group": i})

                    iou_filtered_results = [iou_filter(boxes_metric, iou_idx=get_indices_of_iou(coco)) for boxes_metric
                                            in
                                            fold_filtered_boxes_metrics]
                    scores, _ = coco.compute(iou_filtered_results)
                    for score in scores:
                        df.append({"Metric": score, "Score": scores[score], "Group": i})
            df = pd.DataFrame.from_records(df)
            df.to_excel(writer, sheet_name="Object Detection-{}".format(class_name))

            iou_filtered_results = [iou_filter(boxes_metric, iou_idx=get_indices_of_iou(histo)) for boxes_metric in
                                    fold_filtered_boxes_metrics]
            _, _ = histo.compute(iou_filtered_results)

    df_seg.to_excel(writer, sheet_name="Segmentation")

    writer.close()


if __name__ == "__main__":
    main()



#!/usr/bin/env python

import datetime
from argparse import ArgumentParser, RawTextHelpFormatter
from pathlib import Path
from textwrap import dedent

from PyMAIA.utils.log_utils import get_logger, add_verbosity_options_to_argparser, log_lvl_from_verbosity_args

TIMESTAMP = "{:%Y-%m-%d_%H-%M-%S}".format(datetime.datetime.now())

DESC = dedent(
    """
    Script Description
    """  # noqa: E501
)
EPILOG = dedent(
    """
    Example call:
    ::
        {filename} Script usage examples.
    """.format(  # noqa: E501
        filename=Path(__file__).stem
    )
)


def main():
    parser = get_arg_parser()

    arguments = vars(parser.parse_args())

    logger = get_logger(
        name=Path(__file__).name,
        level=log_lvl_from_verbosity_args(arguments),
    )


def get_arg_parser():
    pars = ArgumentParser(description=DESC, epilog=EPILOG, formatter_class=RawTextHelpFormatter)

    pars.add_argument(
        "-arg",
        "--argument",
        type=str,
        required=True,
        help="Example argument",
    )

    add_verbosity_options_to_argparser(pars)

    return pars


if __name__ == "__main__":
    main()



import json
import os
import random
import shutil
from distutils.dir_util import copy_tree
from multiprocessing import Pool
from os import PathLike
from pathlib import Path
from typing import Union, List, Tuple, Dict

import SimpleITK
import nibabel as nib
import numpy as np
import pydicom
import pydicom_seg
from tqdm import tqdm

from PyMAIA.utils.log_utils import get_logger, DEBUG, WARN, INFO

logger = get_logger(__name__)


def subfiles(
        folder: Union[str, PathLike], join: bool = True, prefix: str = None, suffix: str = None, sort: bool = True
) -> List[str]:
    """
    Given a folder path, returns a list with all the files in the folder.

    Parameters
    ----------
    folder :
        Folder path.
    join :
        Flag to return the complete file paths or only the relative file names.
    prefix :
        Filter the files with the specified prefix.
    suffix :
        Filter the files with the specified suffix.
    sort :
        Flag to sort the files in the list by alphabetical order.

    Returns
    -------
        Filename list.
    """
    if join:
        l = os.path.join  # noqa: E741
    else:
        l = lambda x, y: y  # noqa: E741, E731
    res = [
        l(folder, i.name)
        for i in Path(folder).iterdir()
        if i.is_file() and (prefix is None or i.name.startswith(prefix)) and (suffix is None or i.name.endswith(suffix))
    ]
    if sort:
        res.sort()
    return res


def subfolders(folder: Union[str, PathLike], join: bool = True, sort: bool = True) -> List[str]:
    """
     Given a folder path, returns a list with all the subfolders in the folder.

    Parameters
    ----------
    folder :
        Folder path.
    join :
        Flag to return the complete folder paths or only the relative folder names.
    sort :
        Flag to sort the sub folders in the list by alphabetical order.

    Returns
    -------
        Sub folder list.
    """
    if join:
        l = os.path.join  # noqa: E741
    else:
        l = lambda x, y: y  # noqa: E741, E731
    res = [l(folder, i.name) for i in Path(folder).iterdir() if i.is_dir()]
    if sort:
        res.sort()
    return res


def create_nnunet_data_folder_tree(data_folder: str, task_name: str, task_id: str):
    """
    Create nnUnet folder tree, ready to be populated with the dataset.

    nnUnet folder tree:

        [raw_data_base]

        [Dataset000_Example]
            - dataset.yaml # dataset.json works too
            
            [imagesTr]
                - case0000_0000.nii.gz # case0000 modality 0
                - case0000_0001.nii.gz # case0000 modality 1
                - case0001_0000.nii.gz # case0001 modality 0
                - case0000_0001.nii.gz # case0001 modality 1
                
            [labelsTr]
                - case0000.nii.gz # instance segmentation case0000
                - case0000.json # properties of case0000
                - case0001.nii.gz # instance segmentation case0001
                - case0001.json # properties of case0001
                
            [imagesTs] # optional, same structure as imagesTr
            
            ...
            
            [labelsTs] # optional, same structure as labelsTr

        [Dataset001_Example1]
            ...


    Parameters
    ----------
    data_folder :
        folder path corresponding to the *raw_data_base* environment variable.
    task_name :
        string used as task_name when creating task folder
    task_id :
        string used as task_id when creating task folder
    """  # noqa E501
    logger.log(DEBUG, ' Creating Dataset tree at "{}"'.format(data_folder))

    Path(data_folder).joinpath("nnUNet_raw_data", "Dataset" + task_id + "_" + task_name, "imagesTr", ).mkdir(
        parents=True,
        exist_ok=True,
    )

    Path(data_folder).joinpath("nnUNet_raw_data", "Dataset" + task_id + "_" + task_name, "labelsTr", ).mkdir(
        parents=True,
        exist_ok=True,
    )

    Path(data_folder).joinpath("nnUNet_raw_data", "Dataset" + task_id + "_" + task_name, "imagesTs", ).mkdir(
        parents=True,
        exist_ok=True,
    )

    Path(data_folder).joinpath("nnUNet_raw_data", "Dataset" + task_id + "_" + task_name, "labelsTs", ).mkdir(
        parents=True,
        exist_ok=True,
    )


def create_nndet_data_folder_tree(data_folder: Union[str, PathLike], task_name: str, task_id: str):
    """
    Create nnDetection folder tree, ready to be populated with the dataset.

    nnDetection folder tree:

        [raw_data_base]

        [Task000_Example]
            - dataset.yaml # dataset.json works too

            [raw_splitted]
                [imagesTr]
                    - case0000_0000.nii.gz # case0000 modality 0
                    - case0000_0001.nii.gz # case0000 modality 1
                    - case0001_0000.nii.gz # case0001 modality 0
                    - case0000_0001.nii.gz # case0001 modality 1

                [labelsTr]
                    - case0000.nii.gz # instance segmentation case0000
                    - case0000.json # properties of case0000
                    - case0001.nii.gz # instance segmentation case0001
                    - case0001.json # properties of case0001

                [imagesTs] # optional, same structure as imagesTr
                ...

                [labelsTs] # optional, same structure as labelsTr
                ...

            [preprocessed]

            [results]

        [Task001_Example1]
            ...


    Parameters
    ----------
    data_folder :
        folder path corresponding to the *raw_data_base* environment variable.
    task_name :
        string used as task_name when creating task folder
    task_id :
        string used as task_id when creating task folder
    """
    logger.log(DEBUG, ' Creating Dataset tree at "{}"'.format(data_folder))

    Path(data_folder).joinpath("Task" + task_id + "_" + task_name, "raw_splitted", "imagesTr", ).mkdir(
        parents=True,
        exist_ok=True,
    )

    Path(data_folder).joinpath("Task" + task_id + "_" + task_name, "raw_splitted", "labelsTr", ).mkdir(
        parents=True,
        exist_ok=True,
    )

    Path(data_folder).joinpath("Task" + task_id + "_" + task_name, "raw_splitted", "imagesTs", ).mkdir(
        parents=True,
        exist_ok=True,
    )

    Path(data_folder).joinpath("Task" + task_id + "_" + task_name, "raw_splitted", "labelsTs", ).mkdir(
        parents=True,
        exist_ok=True,
    )


def split_dataset(input_data_folder: Union[str, PathLike], test_split_ratio: int, seed: int,
                  patient_class_file: Union[str, PathLike] = None,
                  classes: List[str] = None) -> Tuple[
    List[str], List[str]]:
    """
    Split dataset into a train/test split, given the specified ratio.

    Parameters
    ----------
    classes :
        List of Patient classes to include in the experiment.
    patient_class_file  :
        File path to JSON Patient Class map.
    input_data_folder :
        folder path of the input dataset.
    test_split_ratio :
        integer value in the range 0-100, specifying the split ratio to be used for the test set.
    seed :
        integer value to be used as random seed.

    Returns
    -------
        lists of strings containing subject IDs for train set and test set respectively.
    """
    subjects = subfolders(input_data_folder, join=False)

    if patient_class_file is not None:
        with open(patient_class_file, "r") as file:
            subject_class = json.load(file)

        if classes is not None:
            filtered_subjects = []
            for subject in subjects:
                if subject_class[subject] in classes and subject != "PETCT_0223010e46_0":
                    filtered_subjects.append(subject)
            subjects = filtered_subjects

    random.seed(seed)
    random.shuffle(subjects)

    split_index = len(subjects) - int(len(subjects) * test_split_ratio / 100)

    train_subjects = subjects[0:split_index]
    test_subjects = subjects[split_index:]

    return train_subjects, test_subjects


def copy_image_file(input_filepath: Union[str, PathLike], output_filepath: Union[str, PathLike]):
    """
    Copy image file.

    Parameters
    ----------
    input_filepath :
        file path for the file to copy
    output_filepath :
        file path where to copy the file
    """
    shutil.copy(
        input_filepath,
        output_filepath,
    )


def copy_label_file(input_image: Union[str, PathLike], input_label: Union[str, PathLike],
                    output_filepath: Union[str, PathLike]):
    """
    Copy label file, verifying the image information (spacing, orientation).

    Parameters
    ----------
    input_image :
        file path for the input image, to be used as reference when copying image information
    input_label :
        file path for the input label to be copied
    output_filepath :
        file location where to save the label image
    """
    label_nib = nib.load(input_label)
    image_nib = nib.load(input_image)

    label_nib_out = nib.Nifti1Image(label_nib.get_fdata(), image_nib.affine)
    nib.save(label_nib_out, output_filepath)


def copy_data_from_dict_to_dataset_folder(
        input_data_file: Union[str, PathLike],
        step: str,
        image_folder: Union[str, PathLike],
        config_dict: Dict[str, object],
        label_folder: Union[str, PathLike] = None,
        num_threads: int = None,
        save_label_instance_config: bool = False,
):
    """

    Parameters
    ----------

    input_data_file :
        JSON dict path of the input dataset
    step :
        string list for the step of the dataset (train/test).
    image_folder :
        folder path where to store images (imagesTr/imagesTs).
    config_dict :
        dictionary with dataset and experiment configuration parameters.
    label_folder :
        folder path where to store labels (labelsTr/labelsTs). Default: ``None``.
        If **label_suffix** is ``None``, the label files are not saved.
    num_threads :
        number of threads to use in multiprocessing ( Default: ``os.environ['N_THREADS']`` )
    save_label_instance_config :
        Flag to save label mask together with an instance dictionary as JSON file. NOTE: All the instances are assigned
         to instance class ``1``.
    """
    label_suffix = str(config_dict["label_suffix"])
    if num_threads is None:
        try:
            num_threads = int(os.environ["N_THREADS"])
        except KeyError:
            logger.warning("N_THREADS is not set as environment variable. Using Default [1]")
            num_threads = 1

    with open(input_data_file, "r") as file:
        input_data_dict = json.load(file)

    pool = Pool(num_threads)
    copied_files = []
    for subject in input_data_dict[step]:

        image_suffix_list = config_dict["Modalities"].keys()

        for modality, image_suffix in enumerate(image_suffix_list):
            modality_code = "_{0:04d}".format(modality)
            image_filename = subject[config_dict["Modalities"][image_suffix]]

            if Path(image_filename).is_file():
                updated_image_filename = Path(image_filename).name.replace(image_suffix,
                                                                           modality_code + str(
                                                                               config_dict["FileExtension"]))
                copied_files.append(
                    pool.starmap_async(
                        copy_image_file,
                        (
                            (
                                str(image_filename),
                                str(Path(image_folder).joinpath(updated_image_filename)),
                            ),
                        ),
                    )
                )
            else:
                logger.warning("{} is not found: skipping {} case".format(image_filename, image_filename))
        if "label" in subject and label_suffix is not None:

            label_filename = subject["label"]

            if Path(label_filename).is_file():

                updated_label_filename = Path(label_filename).name.replace(label_suffix,
                                                                           str(config_dict["FileExtension"]))

                copied_files.append(
                    pool.starmap_async(
                        copy_label_file,
                        (
                            (
                                str(image_filename),
                                str(label_filename),
                                str(Path(label_folder).joinpath(updated_label_filename)),
                            ),
                        ),
                    )
                )
                if save_label_instance_config:
                    label_map = SimpleITK.GetArrayFromImage(
                        SimpleITK.ReadImage(str(Path(label_filename)))
                    )
                    instances = np.unique(label_map)
                    instances = instances[instances > 0]

                    json_dict = {
                        "instances": {str(int(i)): 0 for i in instances},
                    }
                    save_config_json(json_dict,
                                     str(Path(label_folder).joinpath(
                                         Path(label_filename).name.replace(label_suffix, ".json"))))
            else:
                logger.warning("{} is not found: skipping {} case".format(label_filename, label_filename))

    _ = [i.get() for i in tqdm(copied_files)]

def copy_data_to_dataset_folder(
        input_data_folder: Union[str, PathLike],
        subjects: List[str],
        image_folder: Union[str, PathLike],
        config_dict: Dict[str, object],
        label_folder: Union[str, PathLike] = None,
        num_threads: int = None,
        save_label_instance_config: bool = False,
):
    """

    Parameters
    ----------

    input_data_folder :
        folder path of the input dataset
    subjects :
        string list containing subject IDs.
    image_folder :
        folder path where to store images (imagesTr/imagesTs).
    config_dict :
        dictionary with dataset and experiment configuration parameters.
    label_folder :
        folder path where to store labels (labelsTr/labelsTs). Default: ``None``.
        If **label_suffix** is ``None``, the label files are not saved.
    num_threads :
        number of threads to use in multiprocessing ( Default: ``os.environ['N_THREADS']`` )
    save_label_instance_config :
        Flag to save label mask together with an instance dictionary as JSON file. NOTE: All the instances are assigned
         to instance class ``1``.
    """
    label_suffix = str(config_dict["label_suffix"])
    if num_threads is None:
        try:
            num_threads = int(os.environ["N_THREADS"])
        except KeyError:
            logger.warning("N_THREADS is not set as environment variable. Using Default [1]")
            num_threads = 1

    pool = Pool(num_threads)
    copied_files = []
    for directory in subjects:

        files = subfiles(
            str(Path(input_data_folder).joinpath(directory)),
            join=False,
            suffix=str(config_dict["FileExtension"]),
        )

        image_suffix_list = config_dict["Modalities"].keys()

        for modality, image_suffix in enumerate(image_suffix_list):
            modality_code = "_{0:04d}".format(modality)
            image_filename = directory + image_suffix

            if image_filename in files:
                updated_image_filename = image_filename.replace(image_suffix,
                                                                modality_code + str(config_dict["FileExtension"]))
                copied_files.append(
                    pool.starmap_async(
                        copy_image_file,
                        (
                            (
                                str(Path(input_data_folder).joinpath(directory, image_filename)),
                                str(Path(image_folder).joinpath(updated_image_filename)),
                            ),
                        ),
                    )
                )
            else:
                logger.warning("{} is not found: skipping {} case".format(image_filename, directory))
        if label_suffix is not None and label_folder is not None and type(label_suffix) != list:

            label_filename = directory + label_suffix

            if label_filename in files:

                updated_label_filename = label_filename.replace(label_suffix, str(config_dict["FileExtension"]))

                copied_files.append(
                    pool.starmap_async(
                        copy_label_file,
                        (
                            (
                                str(Path(input_data_folder).joinpath(directory, directory + image_suffix)),
                                str(Path(input_data_folder).joinpath(directory, directory + label_suffix)),
                                str(Path(label_folder).joinpath(updated_label_filename)),
                            ),
                        ),
                    )
                )
                if save_label_instance_config:
                    label_map = SimpleITK.GetArrayFromImage(
                        SimpleITK.ReadImage(str(Path(input_data_folder).joinpath(directory, directory + label_suffix)))
                    )
                    instances = np.unique(label_map)
                    instances = instances[instances > 0]

                    json_dict = {
                        "instances": {str(int(i)): 0 for i in instances},
                    }
                    save_config_json(json_dict,
                                     str(Path(label_folder).joinpath(label_filename.replace(label_suffix, ".json"))))
            else:
                logger.warning("{} is not found: skipping {} case".format(label_filename, directory))

        elif type(label_suffix) == list and label_folder is not None:  # Multi Label
            for task_id, label_s in enumerate(label_suffix):
                task_code = "_{0:04d}".format(task_id)
                label_filename = directory + label_s

                if label_filename in files:

                    updated_label_filename = label_filename.replace(label_s,
                                                                    task_code + str(config_dict["FileExtension"]))

                    copied_files.append(
                        pool.starmap_async(
                            copy_label_file,
                            (
                                (
                                    str(Path(input_data_folder).joinpath(directory, directory + image_suffix)),
                                    str(Path(input_data_folder).joinpath(directory, directory + label_s)),
                                    str(Path(label_folder).joinpath(updated_label_filename)),
                                ),
                            ),
                        )
                    )
                else:
                    logger.warning("{} is not found: skipping {} case".format(label_filename, directory))

    _ = [i.get() for i in tqdm(copied_files)]


def save_config_json(config_dict: Dict[str, object], output_json: Union[str, PathLike]):
    """
    Save dictionary as JSON file.

    Parameters
    ----------
    output_json :
        JSON file path to be saved
    config_dict:
        dictionary to be saved in JSON format in the RESULTS_FOLDER
    """

    with open(output_json, "w") as fp:
        json.dump(config_dict, fp)


def generate_dataset_json(
        output_file: Union[str, PathLike],
        train_subjects: List[str],
        test_subjects: List[str],
        modalities: Tuple,
        labels: Union[Dict, List],
        task_name: str,
        file_extension: str,
        region_class_order=None,
        nnunet_format: bool = False,
):
    """
    Generates and saves a Dataset JSON file.

    Parameters
    ----------
    nnunet_format   :
        Flag to specify which modality key to use.
    file_extension  :
        Dataset file extension
    output_file :
        This needs to be the full path to the dataset.json you intend to write, so
    output_file='DATASET_PATH/dataset.json' where the folder DATASET_PATH points to is the one with the
    imagesTr and labelsTr subfolders.
    train_subjects :
        List of subjects in the train set.
    test_subjects :
        List of subjects in the test set.
    modalities :
        tuple of strings with modality names. must be in the same order as the images (first entry
    corresponds to _0000.nii.gz, etc). Example: ('T1', 'T2', 'FLAIR').
    labels :
        dict with int->str (key->value) mapping the label IDs to label names. Note that 0 is always
    supposed to be background! Example: {0: 'background', 1: 'edema', 2: 'enhancing tumor'}. In case of a multi label task,
        the dictionaries for each label task are nested into a list.
    task_name :
        The name of the dataset.
    region_class_order :
        Optional list of strings with the region class order, used for region-based training. Default: ``None``.
    """
    modality_key = "modalities"
    if nnunet_format:
        modality_key = "channel_names"
    json_dict = {
        "task": task_name,
        "dim": 3,
        "test_labels": True,
        "tensorImageSize": "4D",
        modality_key: {str(i): modalities[i] for i in range(len(modalities))},
        "labels": labels,  # {str(i): labels[i] for i in labels.keys()},
        "numTraining": len(train_subjects),
        "numTest": len(test_subjects),
        "training": [{"image": "./imagesTr/%s.nii.gz" % i, "label": "./labelsTr/%s.nii.gz" % i} for i in
                     train_subjects],
        "test": ["./imagesTs/%s.nii.gz" % i for i in test_subjects],
        "file_ending": file_extension,
    }
    if region_class_order:
        json_dict["regions_class_order"] = region_class_order

    if not str(output_file).endswith("dataset.json"):
        print(
            "WARNING: output file name is not dataset.json! This may be intentional or not. You decide. "  # noqa: E501
            "Proceeding anyways..."
        )
    save_config_json(json_dict, output_file)


def remove_empty_folder_recursive(folder_path: Union[str, PathLike]):
    """
    Recursively removes all the empty subdirectories of the root folder.

    Parameters
    ----------
    folder_path :
        Root folder path.
    """
    for subfolder_path in Path(folder_path).glob("*"):
        if Path(subfolder_path).is_dir():
            try:
                os.rmdir(subfolder_path)
            except FileNotFoundError as e:
                logger.log(WARN, e)
            except OSError as e:
                logger.log(WARN, e)
                remove_empty_folder_recursive(subfolder_path)
                os.rmdir(subfolder_path)


def order_data_in_single_folder(
        root_path: Union[str, PathLike],
        output_path: Union[str, PathLike],
        assign_parent_dir_name: bool = False,
        file_extension: str = "",
):
    """
    Moves all the sub-files, found iteratively from the root directory, to the output folder.
    Recursively removes all the empty subdirectories.
    If the *assign_parent_dir_name* flag is set to True, the parent directory name for each file will be used as suffix
    appended to the filename (used when images and masks are divided in different subfolders).

    Parameters
    ----------
    file_extension  :
        File extension for the files in the selected folder.
    assign_parent_dir_name  :
        Flag to set if to assign the parent directory name as suffix.
    root_path   :
        Root folder.
    output_path :
        Output folder.
    """
    logger.log(DEBUG, "Creating folder at '{}'".format(output_path))
    search_regex = "*/*"
    if assign_parent_dir_name:
        search_regex = "*/*/*"

    for file_path in Path(root_path).glob(search_regex):
        if assign_parent_dir_name:

            logger.log(
                DEBUG,
                "Moving '{}' file to '{}'".format(
                    file_path,
                    Path(output_path).joinpath(
                        str(Path(file_path).name[: -len(file_extension)])
                        + "_"
                        + str(Path(file_path).parent.name)
                        + file_extension
                    ),
                ),
            )
            Path(file_path).rename(
                Path(output_path).joinpath(
                    str(Path(file_path).name[: -len(file_extension)]) + "_" + str(
                        Path(file_path).parent.name) + file_extension
                )
            )
        else:
            logger.log(DEBUG,
                       "Moving '{}' file to '{}'".format(file_path, Path(output_path).joinpath(Path(file_path).name)))
            Path(file_path).rename(Path(output_path).joinpath(Path(file_path).name))
    remove_empty_folder_recursive(root_path)


def order_data_folder_by_patient(folder_path: Union[str, PathLike], file_pattern: str):
    """
    Order all the files in the root folder into corresponding subdirectories, according to the specified
    file pattern.

    Parameters
    ----------
    folder_path :
        Root folder path.
    file_pattern    :
        File pattern to group the files and create the corresponding subdirectories.
    """
    patient_id_list = []
    for file_path in Path(folder_path).glob("*"):
        if Path(file_path).is_file() and str(file_path).endswith(file_pattern):
            patient_id_list.append(str(file_path.name)[: -len(file_pattern)])

    logger.log(INFO, "Patient folders in database: {}".format(len(patient_id_list)))

    for patient_id in patient_id_list:
        logger.log(DEBUG, "Creating folder at '{}'".format(Path(folder_path).joinpath(patient_id)))
        Path(folder_path).joinpath(patient_id).mkdir(exist_ok=True, parents=True)

    for file_path in Path(folder_path).glob("*"):
        if Path(file_path).is_file():
            for patient_id in patient_id_list:

                if file_path.name.startswith(patient_id):
                    logger.log(
                        DEBUG,
                        "Moving '{}' file to '{}'".format(
                            file_path, Path(folder_path).joinpath(patient_id, Path(file_path).name)
                        ),
                    )
                    Path(file_path).rename(Path(folder_path).joinpath(patient_id, Path(file_path).name))


def copy_subject_folder_to_data_folder(
        input_data_folder: Union[str, PathLike], subjects: List[str], data_folder: Union[str, PathLike]
):
    """
    Copy all the specified subject sub-folders to a new data folder.

    Parameters
    ----------
    input_data_folder :
        Input data folder.
    subjects    :
        Subjects to copy.
    data_folder :
        Destination data folder.
    """
    Path(data_folder).mkdir(parents=True, exist_ok=True)
    for subject in subjects:
        if Path(input_data_folder).joinpath(subject).is_dir():
            logger.log(DEBUG, "Copying Subject {}".format(subject))
            copy_tree(str(Path(input_data_folder).joinpath(subject)), str(Path(data_folder).joinpath(subject)))


def convert_nifti_pred_to_dicom_seg(
        nifti_pred_file: Union[str, PathLike],
        patient_dicom_folder: Union[str, PathLike],
        template_file: Union[str, PathLike],
        output_dicom_seg: Union[str, PathLike],
        study_id,
):
    """
    Convert a NIFTI prediction file (segmentation mask), into a single DICOM SEG file. ``patient_dicom_folder`` and
    ``template_file`` are used to extract DICOM metadata and information to use when saving the DICOM SEG file.

    Parameters
    ----------
    study_id    :
        Study ID used to match the appropriate DICOM folder.
    nifti_pred_file :
        NIFTI prediction file (segmentation mask) to convert.
    patient_dicom_folder :
        Original patient DICOM folder, used to retrieve DICOM Metadata.
    template_file :
        Template JSON file for the prediction model/algorithm used. Generated from : http://qiicr.org/dcmqi/#/home
    output_dicom_seg :
        Output DICOM SEG file to save.
    """
    segmentation = SimpleITK.ReadImage(nifti_pred_file)
    reader = SimpleITK.ImageSeriesReader()

    studies = subfolders(Path(patient_dicom_folder), join=False)

    dcm_files = None
    for study in studies:
        series = subfolders(Path(patient_dicom_folder).joinpath(studies[0]), join=False)
        for serie in series:
            first_file = next(Path(patient_dicom_folder).joinpath(studies[0], serie).glob("*.dcm"))
            ds = pydicom.dcmread(str(first_file))
            if ds.Modality != "SEG" and ds.StudyInstanceUID == study_id and dcm_files is None:
                dcm_files = reader.GetGDCMSeriesFileNames(str(Path(patient_dicom_folder).joinpath(study, serie)))

    template = pydicom_seg.template.from_dcmqi_metainfo(template_file)
    writer = pydicom_seg.MultiClassWriter(
        template=template,
        inplane_cropping=False,
        skip_empty_slices=False,
        skip_missing_segment=False,
    )

    source_images = [pydicom.dcmread(x, stop_before_pixels=True) for x in dcm_files]
    dcm = writer.write(segmentation, source_images)

    dcm.save_as(output_dicom_seg)



#!/usr/bin/env python

import datetime
import json
import random
import shutil
from argparse import ArgumentParser, RawTextHelpFormatter
from pathlib import Path
from textwrap import dedent

from PyMAIA.utils.log_utils import add_verbosity_options_to_argparser

TIMESTAMP = "{:%Y-%m-%d_%H-%M-%S}".format(datetime.datetime.now())

DESC = dedent(
    """
    Generates and saves a subset, given a dataset. The subset data are extracted from the original dataset according to the
    provided ``classes``. A JSON file mapping each subject to the corresponding class is needed ( ``data_class_file``).
    An optional parameter ``max_size`` can be specified to limit the size of the subset.
    """  # noqa: E501
)
EPILOG = dedent(
    """
    {filename} --data-folder /PATH/TO/DATASET --output-folder /PATH/TO/SUBSET --data-class-file /PATH/TO/SUBJECT_CLASSES.json --subclasses CLASS_1
    {filename} --data-folder /PATH/TO/DATASET --output-folder /PATH/TO/SUBSET --data-class-file /PATH/TO/SUBJECT_CLASSES.json --subclasses CLASS_1 --max-size 100
    """.format(  # noqa: E501
        filename=Path(__file__).stem
    )
)


def main():
    parser = get_arg_parser()

    arguments = vars(parser.parse_args())

    with open(arguments["data_class_file"], "r") as fp:
        data_class_dict = json.load(fp)

    count = 0

    classes = arguments["subclasses"]
    max_size = arguments["max_size"]
    Path(arguments["output_folder"]).mkdir(parents=True, exist_ok=True)
    if max_size is None:
        max_size = len(data_class_dict)

    patients = list(data_class_dict.keys())

    random.shuffle(patients)

    for patient in patients:
        if data_class_dict[patient] in classes and count <= int(max_size):
            if Path(arguments["output_folder"]).joinpath(patient).is_dir():
                ...
            else:
                shutil.copytree(
                    Path(arguments["data_folder"]).joinpath(patient), Path(arguments["output_folder"]).joinpath(patient)
                )
            count += 1


def get_arg_parser():
    pars = ArgumentParser(description=DESC, formatter_class=RawTextHelpFormatter)

    pars.add_argument(
        "--data-folder",
        type=str,
        required=True,
        help="Input dataset folder",
    )

    pars.add_argument(
        "--output-folder",
        type=str,
        required=True,
        help="Output subset folder",
    )

    pars.add_argument(
        "--data-class-file",
        type=str,
        required=True,
        help="JSON file including the class label for each volume in the dataset.",
    )

    pars.add_argument(
        "--subclasses",
        type=str,
        nargs="+",
        required=True,
        help="List of classes from where to select the subset data.",
    )

    pars.add_argument(
        "--max-size",
        type=str,
        required=False,
        help="Maximum size of the generated subset. Default ``None```: no size limit is set.",
    )

    add_verbosity_options_to_argparser(pars)

    return pars


if __name__ == "__main__":
    main()



#!/usr/bin/env python

import json
import os
import subprocess
from argparse import ArgumentParser, RawTextHelpFormatter
from pathlib import Path
from textwrap import dedent

from PyMAIA.utils.log_utils import get_logger, add_verbosity_options_to_argparser, log_lvl_from_verbosity_args, str2bool

DESC = dedent(
    """
    Run ``nndet_train`` command to start nnDetection training for the specified fold.
    """  # noqa: E501
)
EPILOG = dedent(
    """
    Example call:
    ::
        {filename} --config-file ../CONFIG_FILE.json --run-fold 0
        {filename} --config-file ../CONFIG_FILE.json --run-fold 0 --resume-training y
    """.format(  # noqa: E501
        filename=Path(__file__).stem
    )
)


def get_arg_parser():
    pars = ArgumentParser(description=DESC, epilog=EPILOG, formatter_class=RawTextHelpFormatter)

    pars.add_argument(
        "--config-file",
        type=str,
        required=True,
        help="File path for the configuration dictionary, used to retrieve experiments variables (Task_ID)",
    )

    pars.add_argument(
        "--train-config",
        type=str,
        required=True,
        help="File path for the nnDetection Training YAML configuration dictionary, used to configure the nnDetection Experiment.",
    )

    pars.add_argument(
        "--run-fold",
        type=str,
        default="0",
        required=False,
        help="Index indicating which fold to run. Default: ``0``",
    )

    pars.add_argument(
        "--n-folds",
        type=str,
        default="5",
        required=False,
        help="Number of Folds for final consolidation. Default: ``5``",
    )

    pars.add_argument(
        "--model",
        type=str,
        default="RetinaUNetV001_D3V001_3d",
        required=False,
        help="nnDetection Model used for sweeping and consolidation. Default: ``RetinaUNetV001_D3V001_3d``",
    )

    pars.add_argument(
        "--resume-training",
        type=str2bool,
        default="no",
        help="Flag to indicate training resume after stopping it. Default ``no``.",
    )

    pars.add_argument(
        "--n-workers",
        type=str,
        default=None,
        help="Number of parallel processes used when pre-processing and unpacking the image data (Default: ``N_THREADS``)",
    )

    add_verbosity_options_to_argparser(pars)

    return pars


def main():
    parser = get_arg_parser()

    arguments, unknown_arguments = parser.parse_known_args()
    args = vars(arguments)

    logger = get_logger(  # NOQA: F841
        name=Path(__file__).name,
        level=log_lvl_from_verbosity_args(args),
    )

    config_file = args["config_file"]

    with open(config_file) as json_file:
        data = json.load(json_file)

        arguments = [
            "nndet_train",
            data["Task_ID"],
            "-o",
            "exp.fold={}".format(args["run_fold"]),
            "--train-config",
            args["train_config"]
        ]

        if args["resume_training"]:
            arguments.append("-o")
            arguments.append("train.mode=resume")

        arguments.extend(unknown_arguments)

        if not "N_THREADS" in os.environ:
            os.environ["N_THREADS"] = str(os.cpu_count())
        n_workers = "1"
        if args["n_workers"] is None:
            if "N_THREADS" in os.environ is not None:
                n_workers = str(os.environ["N_THREADS"])
        else:
            n_workers = str(args["n_workers"])

        os.environ["det_data"] = data["base_folder"]
        os.environ["OMP_NUM_THREADS"] = "1"
        os.environ["det_num_threads"] = n_workers
        os.environ["nnUNet_def_n_proc"] = n_workers
        os.environ["det_models"] = data["results_folder"]
        os.environ["global_preprocessing_folder"] = data["preprocessing_folder"]

        if int(args["run_fold"]) >= 0:
            subprocess.run(arguments)
            subprocess.run(["nndet_sweep", data["Task_ID"], args["model"], str(args["run_fold"])])

        else:
            subprocess.run(
                ["nndet_consolidate", data["Task_ID"], args["model"], "--sweep_boxes", "--num_folds", args["n_folds"]])
            subprocess.run(
                ["nndet_seg2nii", data["Task_ID"], args["model"], "--fold", str(args["run_fold"])])
            subprocess.run(
                ["nndet_boxes2nii", data["Task_ID"], args["model"], "--fold", str(args["run_fold"])])

            subprocess.run(
                ["nndet_eval", data["Task_ID"], args["model"], str(args["run_fold"]), "--boxes", "--seg",
                 "--analyze_boxes"])


if __name__ == "__main__":
    main()



import os

import setuptools
from setuptools import setup

import versioneer


def resolve_requirements(file):
    requirements = []
    with open(file) as f:
        req = f.read().splitlines()
        for r in req:
            if r.startswith("-r"):
                requirements += resolve_requirements(os.path.join(os.path.dirname(file), r.split(" ")[1]))
            else:
                requirements.append(r)
    return requirements


def read_file(file):
    with open(file) as f:
        content = f.read()
    return content


setup(
    version=versioneer.get_version(),
    packages=setuptools.find_packages(),
    package_data={
        "": ["configs/*.yml", "configs/*.json"],
    },
    zip_safe=False,
    data_files=[('', ["requirements.txt"]), ],
    # package_dir={"": "src"},
    # install_requires=resolve_requirements(os.path.join(os.path.dirname(__file__), "requirements.txt")),

    cmdclass=versioneer.get_cmdclass(),
    entry_points={
        "console_scripts": [
            "PyMAIA_convert_DICOM_dataset_to_NIFTI_dataset = PyMAIA_scripts.PyMAIA_convert_DICOM_dataset_to_NIFTI_dataset:main",
            "PyMAIA_run_pipeline_from_file = PyMAIA_scripts.PyMAIA_run_pipeline_from_file:main",
            "PyMAIA_convert_NIFTI_predictions_to_DICOM_SEG = PyMAIA_scripts.PyMAIA_convert_NIFTI_predictions_to_DICOM_SEG:main",
            "PyMAIA_convert_semantic_to_instance_segmentation = PyMAIA_scripts.PyMAIA_convert_semantic_to_instance_segmentation:main",
            "PyMAIA_create_subset = PyMAIA_scripts.PyMAIA_create_subset:main",
            "PyMAIA_order_data_folder = PyMAIA_scripts.PyMAIA_order_data_folder:main",
            "PyMAIA_downsample_modality = PyMAIA_scripts.PyMAIA_downsample_modality:main",

            "nndet_create_pipeline = PyMAIA_scripts.nndet_create_pipeline:main",
            "nndet_prepare_data_folder = PyMAIA_scripts.nndet_prepare_data_folder:main",
            "nndet_run_preprocessing = PyMAIA_scripts.nndet_run_preprocessing:main",
            "nndet_run_training = PyMAIA_scripts.nndet_run_training:main",
            "nndet_extract_experiment_predictions = PyMAIA_scripts.nndet_extract_experiment_predictions:main",
            "nndet_compute_metric_results = PyMAIA_scripts.nndet_compute_metric_results:main",

            "nnunet_prepare_data_folder = PyMAIA_scripts.nnunet_prepare_data_folder:main",
            "nnunet_run_preprocessing = PyMAIA_scripts.nnunet_run_preprocessing:main",
            "nnunet_run_plan_and_preprocessing = PyMAIA_scripts.nnunet_run_plan_and_preprocessing:main",
            "nnunet_run_training = PyMAIA_scripts.nnunet_run_training:main",
            "nnunet_create_pipeline = PyMAIA_scripts.nnunet_create_pipeline:main",
        ],
    },
    keywords=["deep learning", "image segmentation", "medical image analysis", "medical image segmentation",
              "object detection"],
    # PyMAIA_scripts=glob.glob("PyMAIA_scripts/*"),
)



#!/usr/bin/env python

import datetime
import importlib.resources
import json
import logging
import os
from argparse import ArgumentParser, RawTextHelpFormatter
from pathlib import Path
from textwrap import dedent

import PyMAIA.configs
from PyMAIA.utils.log_utils import (
    get_logger,
    add_verbosity_options_to_argparser,
    log_lvl_from_verbosity_args,
)

TIMESTAMP = "{:%Y-%m-%d_%H-%M-%S}".format(datetime.datetime.now())

DESC = dedent(
    """
    Run nnDet pipeline:
        Data and Folder Preparation -> Preprocessing -> Training.

    """  # noqa: E501
)
EPILOG = dedent(
    """
    Example call:
    ::
        {filename} -i /path/to/input_data_folder --config-file nnDet_3D_config.json --task-ID 000
        {filename} -i /path/to/input_data_folder --config-file nnDet_3D_config.json --task-ID 000 --test-split 25
    """.format(  # noqa: E501
        filename=Path(__file__).stem
    )
)


def get_arg_parser():
    pars = ArgumentParser(description=DESC, epilog=EPILOG, formatter_class=RawTextHelpFormatter)

    pars.add_argument(
        "-i",
        "--input-data-folder",
        type=str,
        required=True,
        help="Input Dataset folder",
    )

    pars.add_argument(
        "--task-ID",
        type=str,
        required=True,
        help="Task ID used in the folder path tree creation.",
    )

    pars.add_argument(
        "--test-split",
        type=int,
        required=False,
        choices=range(0, 101),
        metavar="[0-100]",
        default=20,
        help="Split value ( in %% ) to create Test set from Dataset (Default: 20)",
    )

    pars.add_argument(
        "--config-file",
        type=str,
        required=True,
        help="Configuration JSON file with experiment and dataset parameters.",
    )

    pars.add_argument(
        "--output-file",
        type=str,
        required=False,
        help="Output TXT file path  where to save the pipeline steps.",
    )

    pars.add_argument(
        "--extra-training-config",
        type=str,
        required=False,
        help="Optional JSON file path with nnDetection training configuration.",
    )

    add_verbosity_options_to_argparser(pars)

    return pars


def run_data_and_folder_preparation_step(arguments):
    try:
        with open(arguments["config_file"]) as json_file:
            config_dict = json.load(json_file)
    except FileNotFoundError:
        with importlib.resources.path(PyMAIA.configs, arguments["config_file"]) as json_path:
            with open(json_path) as json_file:
                config_dict = json.load(json_file)

    args = [
        "nndet_prepare_data_folder",
        "--input-data-folder",
        arguments["input_data_folder"],
        "--task-ID",
        arguments["task_ID"],
        "--task-name",
        config_dict["Experiment Name"],
        "--config-file",
        arguments["config_file"],
        "--test-split",
        str(arguments["test_split"]),
    ]

    return args


def run_preprocessing_step(config_file):
    args = [
        "nndet_run_preprocessing",
        "--config-file",
        config_file,
        "--n-workers",
        os.environ["N_THREADS"],
    ]

    return args


def run_training_step(config_file, folds, extra_params=None):
    arg_list = []
    for fold in folds:
        args = [
            "nndet_run_training",
            "--config-file",
            config_file,
            "--run-fold",
            str(fold),
        ]
        if extra_params is not None:
            for parameter in extra_params:
                args.append("{}".format(parameter))
                args.append("{}".format(extra_params[parameter]))

        arg_list.append(args)
    return arg_list


def main():
    parser = get_arg_parser()

    arguments = vars(parser.parse_args())

    logger = get_logger(  # noqa: F841
        name=Path(__file__).name,
        level=log_lvl_from_verbosity_args(arguments),
    )
    try:
        with open(arguments["config_file"]) as json_file:
            config_dict = json.load(json_file)
    except FileNotFoundError:
        with importlib.resources.path(PyMAIA.configs, arguments["config_file"]) as json_path:
            with open(json_path) as json_file:
                config_dict = json.load(json_file)

    output_json_config_filename = "Task" + arguments["task_ID"] + "_" + config_dict["Experiment Name"] + ".json"
    os.environ["RESULTS_FOLDER"] = str(
        Path(os.environ["ROOT_FOLDER"]).joinpath(
            config_dict["Experiment Name"],
            "Task" + arguments["task_ID"] + "_" + config_dict["Experiment Name"],
            "results",
        )
    )
    if "N_THREADS" not in os.environ:
        os.environ["N_THREADS"] = str(os.cpu_count())

    output_json_config_file = str(Path(os.environ["RESULTS_FOLDER"]).joinpath(output_json_config_filename))

    pipeline_steps = []

    pipeline_steps.append(run_data_and_folder_preparation_step(arguments))
    pipeline_steps.append(run_preprocessing_step(output_json_config_file))

    training_params = None
    if arguments["extra_training_config"] is not None:
        with open(arguments["extra_training_config"], "r") as f:
            training_params = json.load(f)

    [
        pipeline_steps.append(step)
        for step in run_training_step(
        output_json_config_file,
        list(range(config_dict["n_folds"]))
        + [
            -1,
        ],
        extra_params=training_params,
    )
    ]

    Path(os.environ["ROOT_FOLDER"]).joinpath(config_dict["Experiment Name"]).mkdir(exist_ok=True,
                                                                                   parents=True)

    output_file = Path(os.environ["ROOT_FOLDER"]).joinpath(
        config_dict["Experiment Name"], "Task_" + arguments["task_ID"] + "_" + TIMESTAMP + ".txt"
    )

    if arguments["output_file"] is not None:
        output_file = arguments["output_file"]
        Path(output_file).parent.mkdir(parents=True, exist_ok=True)

    pipeline_steps_summary = open(
        output_file,
        "w",
    )
    for it, step in enumerate(pipeline_steps):
        logger.log(logging.INFO, "Step {}: {}".format(it, " ".join(step)))
        pipeline_steps_summary.write(" ".join(step) + "\n")

    pipeline_steps_summary.close()


if __name__ == "__main__":
    main()



#!/usr/bin/env python

import os
import sys
from argparse import ArgumentParser, RawTextHelpFormatter
from textwrap import dedent

from PyMAIA.utils.file_utils import order_data_in_single_folder, order_data_folder_by_patient
from PyMAIA.utils.log_utils import (
    get_logger,
    log_lvl_from_verbosity_args,
    add_verbosity_options_to_argparser,
    str2bool,
)

DESC = dedent(
    """
    Order Dataset folder creating and moving the files in the corresponding patient subdirectories
    Folder structure :
        <output_dir>/<patient_id>/<dataset_patient_files>
    The suffix used to find the patient IDs is set via the --patient-suffix flag:
        Filename -> [patient_ID + patient_suffix]
    Example:
        --patient-suffix _image.nii.gz
        PatientA_image.nii.gz  -> [patient_ID = PatientA]
    If -in-place [yes/no] is set to 'yes', --output-folder is not considered and the folder ordering is performed on the input folder
    """  # noqa: E501
)
EPILOG = dedent(
    """
    Example call:
    ::
        {filename} -i /path/to/inputfolder -o /path/to/outputfolder --patient-suffix _image.nii.gz
        {filename} -i /path/to/inputfolder -o /path/to/outputfolder --patient-suffix _image.nii.gz
        {filename} -i /path/to/inputfolder --patient-suffix _image.nii.gz --in-place yes
    """.format(  # noqa: E501
        filename=os.path.basename(__file__)
    )
)


def get_arg_parser():
    parser = ArgumentParser(description=DESC, epilog=EPILOG, formatter_class=RawTextHelpFormatter)

    parser.add_argument(
        "-i",
        "--input-folder",
        type=str,
        required=True,
        help="Input Dataset folder",
    )

    parser.add_argument(
        "-o",
        "--output-folder",
        type=str,
        required=not ("--in-place" in sys.argv and str2bool(sys.argv[sys.argv.index("--in-place") + 1])),
        help="Output folder where to save the ordered Dataset",
    )

    parser.add_argument(
        "--patient-suffix",
        type=str,
        required=True,
        help="Suffix used to find the patient IDs from the filenames",
    )

    parser.add_argument(
        "--file-extension",
        type=str,
        default=".nii.gz",
        required=False,
        help="File extension for the files in the dataset. (Default: `.nii.gz`)",
    )

    parser.add_argument(
        "--in-place",
        type=str2bool,
        required=False,
        default="no",
        help='if set to "yes", the output folder matches the input folder',
    )

    parser.add_argument(
        "--create-subject-subfolders",
        type=str2bool,
        required=False,
        default="yes",
        help='If set to "yes", individual subject subfolders are created, with the corresponding files stored. If set '
             'to "no", all the files are directly saved in the main data folder.',
    )

    parser.add_argument(
        "--assign-parent-dir-name",
        type=str2bool,
        required=False,
        default="no",
        help='If set to "yes", the parent directory name for all the files will be appended to the filename. To be used when'
             'the files for the same patient are saved in different folders ("image", "mask"), with the same name',
    )

    add_verbosity_options_to_argparser(parser)

    return parser


def main():
    parser = get_arg_parser()
    args = vars(parser.parse_args())

    logger = get_logger(  # noqa: F841
        name=os.path.basename(__file__),
        level=log_lvl_from_verbosity_args(args),
    )

    if args["in_place"]:
        args["output_folder"] = args["input_folder"]

    if args['assign_parent_dir_name']:
        order_data_in_single_folder(args["input_folder"], args["output_folder"], True,
                                    file_extension=args['file_extension'])
    else:
        order_data_in_single_folder(args["input_folder"], args["output_folder"])
    if args["create_subject_subfolders"]:
        order_data_folder_by_patient(args["output_folder"], args["patient_suffix"])


if __name__ == "__main__":
    main()



import argparse
import logging
from argparse import ArgumentParser
from typing import Dict, Union

import coloredlogs

__VERBOSITY_GRANULARITY__ = 2

LOG_FMT = "%(asctime)s - %(name)s - %(levelname)s - %(message)s"

# define custom log levels
DEBUG = logging.DEBUG
DEBUG2 = DEBUG - 2
DEBUG4 = DEBUG - 4
DEBUG6 = DEBUG - 6
WARN = logging.WARN
INFO = logging.INFO
# add levels to the 'DEBUG' group (i.e. same coloring)
for lvl_value in [DEBUG2, DEBUG4, DEBUG6]:
    logging.addLevelName(lvl_value, "DEBUG")


def setup_logging(level: Union[str, int] = "DEBUG", fmt: str = LOG_FMT):
    coloredlogs.install(level=level, fmt=fmt)


def get_logger(name: str = "", level: Union[str, int] = None, fmt: str = LOG_FMT) -> logging.Logger:
    """Get a logger and setup colored logging if level is provided."""

    if level is not None:
        setup_logging(level=level, fmt=fmt)

    logger = logging.getLogger(name)

    return logger


def add_verbosity_options_to_argparser(p: ArgumentParser):
    gp = p.add_mutually_exclusive_group(required=False)
    gp.add_argument(
        "-v",
        "--verbose",
        required=False,
        action="count",
        help="Enable verbose output in terminal. Add multiple times to increase verbosity.",  # noqa: E501
    )
    gp.add_argument(
        "-q",
        "--silent",
        required=False,
        action="count",
        help="Suppress most log outputs in terminal.",
    )


def log_lvl_from_verbosity_args(args: Dict) -> int:
    if args["verbose"]:
        return logging.DEBUG - __VERBOSITY_GRANULARITY__ * (args["verbose"] - 1)  # noqa: 501
    elif args["silent"]:
        return logging.WARN
    else:
        return logging.INFO


def str2bool(v):
    if isinstance(v, bool):
        return v
    if v.lower() in ("yes", "true", "t", "y", "1"):
        return True
    elif v.lower() in ("no", "false", "f", "n", "0"):
        return False
    else:
        raise argparse.ArgumentTypeError("Boolean value expected.")
